//part of design

package kripos.gateway;

import java.util.Date;
import java.util.ArrayList;

import kripos.geo.DelayVector;
import kripos.geo.Landmark;
import kripos.ontology.TraceResult;

/**
 * Class for wrapping all necessary information in a bundle for easy exchange between 
 * the multi-agent system and the gateway-classes.
 *  
 * @author oysteine
 * @version 1.0
 */
public class CommandPackage {
  private String target; //the target of the trace
  private String type; //trace type. may be extended to also cover securing of content etc
  private Date reqTime;// when was the trace requested
  private Date repTime;// when was the trace finalized
  private boolean successful = false; //the final state of the trace
  private ArrayList<Landmark> geoPingResults;
  private ArrayList<TraceResult> CBGResults  = new ArrayList<TraceResult>();
  private ArrayList<DelayVector> delayVectors = new ArrayList<DelayVector>();

  /**
   * Construct an empty CommandPackage. 
   * Any fields must be filled by the relevant set methods.
   */
  public CommandPackage() {
  }

  public void addResult(TraceResult tr){
    CBGResults.add(tr);
  }

  /**
   * 
   * @return the cBGResults
   */
  public ArrayList<TraceResult> getCBGResults() {
    return CBGResults;
  }


  /**
   * @param results the cBGResults to set
   */
  public void setCBGResults(ArrayList<TraceResult> results) {
    CBGResults = results;
  }


  /**
   * @return the geoPingResults
   */
  public ArrayList getGeoPingResults() {
    return geoPingResults;
  }


  /**
   * @param geoPingResults the geoPingResults to set
   */
  public void setGeoPingResults(ArrayList geoPingResults) {
    this.geoPingResults = geoPingResults;
  }


  /**
   * @return the repTime
   */
  public Date getRepTime() {
    return repTime;
  }


  /**
   * @param repTime the repTime to set
   */
  public void setRepTime(Date repTime) {
    this.repTime = repTime;
  }


  /**
   * @return the reqTime
   */
  public Date getReqTime() {
    return reqTime;
  }


  /**
   * @param reqTime the reqTime to set
   */
  public void setReqTime(Date reqTime) {
    this.reqTime = reqTime;
  }


  /**
   * @return the successful
   */
  public boolean isSuccessful() {
    return successful;
  }


  /**
   * @param successful the successful to set
   */
  public void setSuccessful(boolean successful) {
    this.successful = successful;
  }


  /**
   * @return the target
   */
  public String getTarget() {
    return target;
  }


  /**
   * @param target the target to set
   */
  public void setTarget(String target) {
    this.target = target;
  }


  /**
   * @return the type
   */
  public String getType() {
    return type;
  }


  /**
   * @param type the type to set
   */
  public void setType(String type) {
    this.type = type;
  }

  public ArrayList<DelayVector> getDelayVectors(){
    return delayVectors;
  }

}//class
