package kripos.gateway;

import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.Random;
import java.util.concurrent.ConcurrentHashMap;
import jade.content.ContentElement;
import jade.content.lang.Codec.CodecException;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.OntologyException;
import jade.content.onto.UngroundedException;
import jade.content.onto.basic.Action;
import jade.content.onto.basic.Result;
import jade.core.AID;
import jade.core.ContainerID;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.FIPAManagementOntology;
import jade.domain.FIPAAgentManagement.SearchConstraints;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.domain.JADEAgentManagement.WhereIsAgentAction;
import jade.domain.mobility.MobilityOntology;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.ConversationList;
import jade.lang.acl.MessageTemplate;
import jade.wrapper.gateway.GatewayAgent;
import kripos.ontology.InternetInvestigationsOntology;

/**
 * @author oysteine
 * @version 1.0
 *
 */
public class GWAgent extends GatewayAgent {
  private static final long serialVersionUID = -5814890860284035720L;
  protected ConversationList convList;
  protected GregorianCalendar myCal;
  protected ConcurrentHashMap<AID, String> adminAgents = new ConcurrentHashMap<AID, String>();
  private ConcurrentHashMap<String, CommandPackage> activeCommands = 
    new ConcurrentHashMap<String, CommandPackage>();

  /**
   * 
   */
  public GWAgent() {
    convList = new ConversationList(this);
    myCal = new GregorianCalendar(); //timezone, not implemented
  }

  @Override
  protected void setup(){
    getContentManager().registerLanguage(new SLCodec(0));
    getContentManager().registerLanguage(new SLCodec());//some messages from AMS not set to SL-0
    getContentManager().registerOntology(MobilityOntology.getInstance());
    getContentManager().registerOntology(InternetInvestigationsOntology.getInstance());
    getContentManager().registerOntology(FIPAManagementOntology.getInstance());

    //find AdminAgents
    DFAgentDescription adminTemplate = new DFAgentDescription();
    ServiceDescription adminSd = new ServiceDescription();
    adminSd.setType("AdminAgent");
    adminTemplate.addServices(adminSd);
    SearchConstraints sc = new SearchConstraints();
    sc.setMaxDepth(new Long(0));
    sc.setMaxResults(new Long(10000));
    ArrayList<DFAgentDescription> tempList = new ArrayList<DFAgentDescription>();

    try {
      DFAgentDescription dfas[] = DFService.search(this, adminTemplate, sc);
      for(int i=0; i<dfas.length;i++){
        tempList.add(dfas[i]);
      }

    } catch (FIPAException e1) {
      e1.printStackTrace();
    }

    for(int i=0;i< tempList.size();i++){
      WhereIsAgentAction wiaa = new WhereIsAgentAction();
      wiaa.setAgentIdentifier(tempList.get(i).getName());
      AID myAMS = getAMS();
      Action act = new Action(myAMS, wiaa);

      ACLMessage qMsg = new ACLMessage(ACLMessage.REQUEST);
      String convID = getName() + new Random().nextLong();
      qMsg.setConversationId(convID);

      qMsg.setLanguage(new SLCodec(0).getName());
      qMsg.setOntology(MobilityOntology.getInstance().getName());

      try {
        getContentManager().fillContent(qMsg, act);
        qMsg.addReceiver(act.getActor());
        //register conversation with agent to get correct reception
        getConvList().registerConversation(convID);
        send(qMsg);

        MessageTemplate mt = MessageTemplate.MatchConversationId(convID);
        ACLMessage resp = blockingReceive(mt, 1000000);
        getConvList().deregisterConversation(convID);
        ContentElement ce = getContentManager().extractContent(resp);
        Result result = (Result) ce;
        ContainerID cid = (ContainerID)result.getValue();
        adminAgents.put(tempList.get(i).getName(), cid.getName());
      } 
      catch (UngroundedException e) {
        e.printStackTrace();
      } catch (CodecException e) {
        e.printStackTrace();
      } catch (OntologyException e) {
        e.printStackTrace();
      }
    }

    AID df = getDefaultDF();
    ACLMessage adminSubs = DFService.createSubscriptionMessage(this, df, adminTemplate, sc);
    convList.registerConversation(adminSubs.getConversationId());

    addBehaviour(new AdminSubscribeBehaviour(this, adminSubs, adminAgents));

    addBehaviour(new GWReceiveBehaviour(this));
  }

  /**
   * 
   */
  @Override
  protected void processCommand(java.lang.Object command){
    if (command instanceof CommandPackage){ 
      CommandPackage cp = (CommandPackage)command;
      addBehaviour(new LaunchTraceBehave(this, cp));
    }
    else {
      //todo throw exception to alert gw and webapp/user of error
      System.out.println("erereorerer");
      releaseCommand(command);
    }
  }

  /**
   * Get the CommandPackage assosciated with the conversation ID supplied
   * Remove the CommandPackage from the map
   * Deregisters the conversation ID with the agent
   * 
   * @param conversationID to match
   * @return commandpackage assosciated with conversationID
   */
  public CommandPackage getMatchingCommand(String conversationID){
    convList.deregisterConversation(conversationID);
    return activeCommands.remove(conversationID);
  }

  /**
   * Adds an entry in the map of activeCommands
   * 
   * @param convID
   * @param cp
   */
  public void addCommand(String convID, CommandPackage cp){
    activeCommands.put(convID, cp);
  }

  /**
   * Returns the conversationlist of this agent
   * 
   * @return the ConversationList of this agent
   */
  protected ConversationList getConvList(){
    return convList;
  }

  /**
   * Returns the GregorianCalendar of this agent
   * 
   * @return the calendar of this agent
   */
  protected GregorianCalendar getCalendar(){
    return myCal;
  }

}//class