package kripos.gateway;

import java.util.ArrayList;

import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.UngroundedException;
import jade.content.onto.basic.Action;
import jade.core.behaviours.CyclicBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import kripos.ontology.TraceResult;
import kripos.ontology.TraceResultList;
import kripos.ontology.UpdateTime;

/**
 * @author oysteine
 * @version 1.0
 *
 */
public class GWReceiveBehaviour extends CyclicBehaviour {
  private static final long serialVersionUID = 2014289984738025171L;
  private GWAgent myGWAgent;	

  /**
   * @param a
   */
  public GWReceiveBehaviour(GWAgent a) {
    super(a);
    myGWAgent = a;
  }

  /* (non-Javadoc)
   * @see jade.core.behaviours.Behaviour#action()
   */
  @Override
  public void action() {
    MessageTemplate mt = myGWAgent.getConvList().getMessageTemplate();
    ACLMessage msg = myAgent.receive(mt);
    if(msg != null){
      handle(msg);
    }
    block(100); //this SHOULD work without timeout
    //block();
  }

  /**
   * Handles incoming messages. 
   * If the received action is not understood a NOT_UNDERSTOOD messages is returned to the sender.
   * 
   * @param msg incoming {@link jade.lang.acl.ACLMessage}
   */
  private void handle(ACLMessage msg){
    try {
      ContentElement content = myAgent.getContentManager().extractContent(msg);
      Concept action = ((Action)content).getAction();

      if (action instanceof TraceResultList){
        TraceResultList trl = (TraceResultList)action;
        CommandPackage cp = myGWAgent.getMatchingCommand(msg.getConversationId());
        ArrayList<TraceResult> resList = new ArrayList<TraceResult>();
        for(int i=0;i<trl.getTraceResult().size();i++){
          resList.add((TraceResult)trl.getTraceResult().get(i));
        }
        if(cp.getType().equalsIgnoreCase("TRACE-CBG")){
          cp.setCBGResults(resList);

        }
        else if (cp.getType().equalsIgnoreCase("TRACE-GEOPING")){
          cp.setGeoPingResults(resList);
        }

        cp.setRepTime(myGWAgent.getCalendar().getTime());
        if(trl.getDoneStatus().equalsIgnoreCase("OK")){
          cp.setSuccessful(true);}
        else{
          cp.setSuccessful(false);
        }
        myGWAgent.releaseCommand(cp);
      }
      else if (action instanceof UpdateTime){
        UpdateTime  upd = (UpdateTime)((Action)content).getAction();
        long offset = Math.round(upd.getHasOffset());
        myGWAgent.getCalendar().setTimeInMillis(offset + myGWAgent.getCalendar().getTimeInMillis());
        //log to SysLog
      }
      else {
        ACLMessage reply = msg.createReply();
        msg.setPerformative(ACLMessage.NOT_UNDERSTOOD);
        myAgent.send(reply);
      }
    }
    catch (UngroundedException e) {
      e.printStackTrace();
    } 

    catch (CodecException e) {
      e.printStackTrace();
    } 

    catch (OntologyException e) {
      e.printStackTrace();
    }
  }

}//class
