package kripos.geo;

import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.UngroundedException;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import kripos.ontology.LogRec;
import kripos.ontology.OntDate;
import kripos.ontology.TraceResultList;

/**
 * 
 * 
 * @author oysteine
 * @version 1.0
 */
public class AdminReplyGWBehaviour extends OneShotBehaviour {
  private static final long serialVersionUID = 4025250051872250273L;
  private AdminAgent myAdminAgent;
  private ACLMessage myMsg;
  private TraceResultList myTrl;

  /**
   * @param a
   */
  public AdminReplyGWBehaviour(AdminAgent a, ACLMessage msg) {
    super(a);
    myAdminAgent = a;
    myMsg = msg;
  }

  /**
   * Closes the case, logs appropriate information.
   * 
   * @see jade.core.behaviours.Behaviour#action()
   */
  @Override
  public void action() {
    try {
      AID aid = myMsg.getSender();
      ContentElement ce = myAdminAgent.getContentManager().extractContent(myMsg);
      Concept action = ((Action)ce).getAction();
      myTrl = (TraceResultList)action;
      String status = myTrl.getDoneStatus();

      if(status.equalsIgnoreCase("OK")){

        int cID = myAdminAgent.removeSessionAgentPackage(aid);
        myAdminAgent.removeActiveCase(cID);
        ACLMessage reply = myMsg.createReply();
        reply.setPerformative(ACLMessage.INFORM);
        myAgent.send(reply);

        /* Logs that the Case has been closed*/
        log("Case closed. SessionAgent terminated. Case end status: "+ 
            myTrl.getDoneStatus(), "Type:Session", cID);
        //send die to SessionAgent

        //send data to GWAgent
        GWCase gwCase = myAdminAgent.getGWCase(cID);
        AID gwAID = gwCase.getAID();
        String convID = gwCase.getConversationID();

        //use myMsg for sending to GWAgent, already has correct content, only set envelopeinfo
        myMsg.clearAllReceiver();
        myMsg.clearAllReplyTo();
        myMsg.addReceiver(gwAID);
        myMsg.setConversationId(convID);
        myMsg.setSender(myAdminAgent.getAID());
        myAgent.send(myMsg);	            
      }
      else if(status.equalsIgnoreCase("Fail")){
        int cID = myAdminAgent.removeSessionAgentPackage(aid);
        myAdminAgent.removeActiveCase(cID);

        //FIXME logging
        /*
	            log("Case closed. SessionAgent terminated. Case end status: "+ 
	                  myTrl.getDoneStatus() + "\n" + "Case FAILED due to: " 
	                  + myTrl.getHasReason(), "Type:Session", cID);
         */

        //send data to GWAgent
        AID gwAID = myAdminAgent.getGWCase(cID).getAID();
        String convID = myAdminAgent.getGWCase(cID).getConversationID();

        //use myMsg for sending to GWAgent, already has correct content, only set envelopeinfo
        myMsg.clearAllReceiver();
        myMsg.clearAllReplyTo();
        myMsg.addReceiver(gwAID);
        myMsg.setConversationId(convID);
        myMsg.setSender(myAdminAgent.getAID());
        myAgent.send(myMsg);
      }
      else{
        ACLMessage reply = myMsg.createReply();
        reply.setPerformative(ACLMessage.NOT_UNDERSTOOD);
        myAgent.send(reply);
      }

    } catch (UngroundedException e) {
      e.printStackTrace();
    } catch (CodecException e) {
      e.printStackTrace();
    } catch (OntologyException e) {
      e.printStackTrace();
    }
  }//action()

  /**
   * Utility method for constructing LogRecs.
   * 
   * @param logContent The content of the LogRec
   * @param logType The Type of the LogRec
   * @param caseID The CaseID of the LogRec
   */
  private void log(String logContent, String logType, int caseID){
    LogRec logr = new LogRec();
    logr.setHasCaseID(caseID);
    //logr.setHasSessionID(myCase.getCaseSessions().getSessionID());
    logr.setLogType(logType);
    OntDate od = new OntDate();
    od.setTime(myAdminAgent.getCalendar().getTimeInMillis());
    logr.setHasDate(od);
    logr.setLogLevel("INFO");
    logr.setLogContent(logContent);
    myAdminAgent.getLogger().log(logr);
  }

}//class