package kripos.geo;

import java.util.ArrayList;

/**
 * A delay vector for a given landmark.
 * 
 * @author oysteine
 * @version 1.0
 */
public class DelayVector extends ArrayList {
  private static final long serialVersionUID = 7614366734429571963L;
  private String myName;
  private double sum;

  /**
   * 
   * @param name The name of the landmark represented by this instance
   */
  public DelayVector(String name) {
    myName = name;
  }

  /**
   * Adds a delay component to this DelayVector
   * 
   * @param toThis the delay from calling landmark to this landmark
   * @param toTarget the delay from calling landmark to the target 
   */
  public void add(double toThis, double toTarget){
    Double temp = new Double((toThis-toTarget)*(toThis-toTarget));
    this.add(temp);
    sum = sum+temp;
  }

  /**
   * Computes the Euclidian distance of this DelayVector
   * 
   * @return the euclidian distance
   */
  public double euclidianDistance(){
    if (sum>0){
      return Math.sqrt(sum);
    }
    else {
      return -1;
    }
  }

  /**
   * 
   * @return the name of the landmark represented by this DelayVector
   */
  public String getName(){
    return myName;
  }

}//class