package kripos.geo;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Random;
import jade.content.ContentElement;
import jade.content.lang.Codec.CodecException;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.OntologyException;
import jade.content.onto.UngroundedException;
import jade.content.onto.basic.Action;
import jade.content.onto.basic.Result;
import jade.core.AID;
import jade.core.ContainerID;
import jade.core.Location;
import jade.core.behaviours.OneShotBehaviour;
import jade.domain.JADEAgentManagement.CreateAgent;
import jade.domain.JADEAgentManagement.QueryPlatformLocationsAction;
import jade.domain.JADEAgentManagement.WhereIsAgentAction;
import jade.domain.mobility.MobilityOntology;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import kripos.gateway.CommandPackage;
import kripos.ontology.*;

/**
 * Behaviour that initiates a trace
 * 
 * @author oysteine
 * @version 1.0
 */
public class InitiateTraceSessionBehaviour extends OneShotBehaviour {
  private static final long serialVersionUID = -5048721800518980650L;
  //private ArrayList<Location> locations = new ArrayList<Location>();
  private SessionAgent mySessionAgent;
  private Session mySession;
  private boolean disableLookupDB = true;
  private boolean disablePassive = true;

  /**
   * Creates an instance of this behaviour.
   * 
   * @param a The agent this instance belongs to.
   */
  public InitiateTraceSessionBehaviour(SessionAgent a, Session s) {
    super(a);
    mySessionAgent = a;
    mySession = s;
  }

  /**
   * Does initial work based on the type of trace session.
   *    * 
   * @see jade.core.behaviours.OneShotBehaviour#action()
   */	
  @Override
  public void action(){
    OntAddress toTrace = mySession.getHasBaseAddress();
    String params = mySession.getSessionParameters();
    //parse and check if dblookup
    //parse and check if passive
    //parse and check random-period for when to ping at a

    String sessionType = mySession.getSessionType();

    //fill commandpackage in owning SessionAgent
    CommandPackage atr = mySessionAgent.getAccTraceResults();
    atr.setReqTime(mySession.getSessionStarted());
    if(toTrace instanceof IP) {
      atr.setTarget(((IP)toTrace).getHasName());
    }
    else if(toTrace instanceof DNSname){
      atr.setTarget(((DNSname)toTrace).getHasName());
    }
    atr.setType(sessionType);

    //TODO SEND 3 PINGS TO CHECK IF REACHABLE AT ALL BEFORE WASTING RESOURCES

    //TODO db lookup
    if(!disableLookupDB){
      dbLookup(toTrace);
    }

    if(!disablePassive){
//    //TODO must return list of locations to use for trace
      ArrayList locations = passiveLookup(toTrace);
      //TODO superlandmark from this container? 
      createWorkers(locations);
    }
    else {		
      //TODO superlandmark from this container?  
      createWorkers(null);	
    }


//  /* Logs the start of the Session*/ TODO log correct information
//  log("Session started. WorkerAgents created" + "\n" +
//  "SessionType: " + mySession.getSessionType() +
//  "\n" + "Target: "+ mySession.getHasProtocol().getHasPrefix() +
//  mySession.getHasBaseAddress()+mySession.getHasPath().toString(),
//  "Type:Session", mySession.getCaseID());

  }

  /**
   * TODO
   *  
   * @param adr
   * @return
   */
  private boolean dbLookup(OntAddress adr){ //not return true?
    boolean recent = false;
    //les inn terskel fra properties?
    //dboppslag

    //if(){} break and return traceresult directly from db.
    //else{
    return recent;
    //	}
  }

  /**
   * TODO 
   * 
   * @param adr
   * @return
   */
  private ArrayList passiveLookup(OntAddress adr){
    ArrayList list = new ArrayList();
    return list;
  }

  /**
   * Creates the WorkerAgents that will perform the rest of 
   * the delay measurements of the trace operation.
   *  
   * @param onLocations List of Locations to do measurements from. 
   * If null all available Locations are used.
   */
  private void createWorkers(ArrayList onLocations){
    ArrayList<Location> locations = new ArrayList<Location>();
    AID myAMS = myAgent.getAMS();

    //query ams for platform available locations
    QueryPlatformLocationsAction qpla = new QueryPlatformLocationsAction();
    Action actAll = new Action(myAMS, qpla);
    String convIDAll = sendRequest(actAll);

    //receive locations
    Result resAll = receiveInform(convIDAll);
    jade.util.leap.Iterator it = resAll.getItems().iterator();

    //uses all available locations
    if(onLocations == null){
      //due to unclear JadeGateway.init() documentation, need to filter out extra containers at mainhost
      ArrayList<String> onlyInclude = readLandmarks(connectDB()); 

      while (it.hasNext()) {
        Location loc = (Location)it.next();
        for (int i=0;i<onlyInclude.size();i++){
          if(loc.getName().equalsIgnoreCase(onlyInclude.get(i))){
            locations.add(loc);
            break;
          }
        }
      }
//    //add own location
//    WhereIsAgentAction wia = new WhereIsAgentAction();
//    Action actSelf = new Action(myAMS, wia);
//    String convSelf = sendRequest(actSelf);

//    Result resSelf = receiveInform(convSelf);
//    ContainerID cid = (ContainerID)resSelf.getValue();

//    locations.add(cid);
    }

    //uses only supplied locations
    else{
      while (it.hasNext()) {
        Location loc = (Location)it.next();
        for(int i=0;i<onLocations.size();i++){ //should optimize by removing from onLocation if match
          if(loc.getName().equalsIgnoreCase((String)onLocations.get(i))){
            locations.add(loc);
          }
        }
      }
    }

    //actually create agents
    for (int i=0; i < locations.size(); i++){
      CreateAgent ca = new CreateAgent();
      ca.setAgentName("Worker" + mySession.getSessionID() +" "+ i);
      ca.setClassName("kripos.geo.WorkerAgent");
      ContainerID cid = (ContainerID)locations.get(i);
      ca.setContainer(cid);
      ca.addArguments(myAgent.getName());

      WorkItem wi = new WorkItem();
      wi.setCaseID(mySession.getCaseID());
      wi.setSessionID(mySession.getSessionID());
      wi.setHasBaseAddress(mySession.getHasBaseAddress());
      wi.setSessionType(mySession.getSessionType());
      ca.addArguments(wi);			
      Action a2 = new Action(myAMS, ca);
      String convID2 = sendRequest(a2);

      MessageTemplate mt = MessageTemplate.MatchConversationId(convID2);
      ACLMessage resp = myAgent.blockingReceive(mt, 20000);

      mySessionAgent.getConvList().deregisterConversation(resp.getConversationId());
    }

    locations.clear();
  }

  /**
   * Utility method for sending ACLMessages 
   * <p>
   * Registers the conversationID in the convList 
   * to avoid loosing any reply to the generic receivebehaviour 
   * 
   * @param action the action that is to be wrapped in an ACLMessage 
   * @return
   */
  private String sendRequest(Action action){
    try {
      ACLMessage qMsg = new ACLMessage(ACLMessage.REQUEST);
      qMsg.setConversationId(myAgent.getName() + new Random().nextLong());
      qMsg.setLanguage(new SLCodec(0).getName());

      qMsg.setOntology(MobilityOntology.getInstance().getName());
      myAgent.getContentManager().fillContent(qMsg, action);
      qMsg.addReceiver(action.getActor());
      //register conversation with agent to get correct reception
      mySessionAgent.getConvList().registerConversation(qMsg.getConversationId());
      myAgent.send(qMsg);
      return qMsg.getConversationId();
    } 
    catch (CodecException e) {
      e.printStackTrace();
      return null;
    } 
    catch (OntologyException e) {
      e.printStackTrace();
      return null;
    }
  }
  /**
   * Utility method for receiving ACLMessages with mathcing conversation IDs
   * 
   * @param cid the conversation id to match
   * @return the content of the ACLMessage
   */	
  private Result receiveInform(String cid){
    try {
      MessageTemplate mt = MessageTemplate.MatchConversationId(cid);
      ACLMessage resp = myAgent.blockingReceive(mt, 20000);
      ContentElement ce = myAgent.getContentManager().extractContent(resp);
      Result result = (Result) ce;
      mySessionAgent.getConvList().deregisterConversation(cid);
      return result;
    } 
    catch (UngroundedException e) {
      e.printStackTrace();
      mySessionAgent.getConvList().deregisterConversation(cid);
      return null;
    } 
    catch (CodecException e) {
      e.printStackTrace();
      mySessionAgent.getConvList().deregisterConversation(cid);
      return null;
    } 
    catch (OntologyException e) {
      e.printStackTrace();
      mySessionAgent.getConvList().deregisterConversation(cid);
      return null;
    }
  }

  /**
   * Reads landmarks from the local database.
   * 
   * @param con Database connection
   * @return a list of landmarks
   */
  private ArrayList<String> readLandmarks(Connection con){
    ArrayList<String> landmarkNames = new ArrayList<String>();

    try {
      Statement stmt = con.createStatement();
      String query = "SELECT NAME FROM LANDMARKS";
      ResultSet rs = stmt.executeQuery(query);
      while(rs.next()){
        String name = rs.getString("NAME");
        landmarkNames.add(name);
      }
    }catch (SQLException e) {
      // TODO: handle exception
    }
    return landmarkNames;
  }

  /**
   * Connects to the local database and returns the Connection for further use.
   * 
   * @return a connection to the database
   */
  private Connection connectDB(){//TODO path & password
    Connection c = null;
    try {
      Class.forName("org.hsqldb.jdbcDriver");
      c = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "sa", "");
    }catch (ClassNotFoundException e) {
      //FIXME unable to find database classes. dosomething
      e.printStackTrace();
    } 
    catch (SQLException e) {
      //FIXME unable to connect to database dosomething
      e.printStackTrace();
    }
    return c;
  }

  /**
   * Utility method for constructing LogRecs  and logging them.
   * 
   * @param logContent The content of the LogRec
   * @param logType The Type of the LogRec
   * @param caseID The CaseID of the LogRec
   */
  private void log(String logContent, String logType, int caseID){
    LogRec logr = new LogRec();
    logr.setHasCaseID(caseID);
    logr.setHasSessionID(mySession.getSessionID());
    logr.setLogType(logType);
    OntDate od = new OntDate();
    od.setTime(mySessionAgent.getCalendar().getTimeInMillis());
    logr.setHasDate(od);
    logr.setLogLevel("INFO");
    logr.setLogContent(logContent);
    mySessionAgent.getLogger().log(logr);
  }

}//class
