package kripos.geo;

import jade.core.behaviours.OneShotBehaviour;
import kripos.geo.ping.HostUnreachableException;
import kripos.geo.ping.MinRTT;

/**
 * Performs actual pinging of remote hosts.
 * Used by WorkerAgents and SessionAgents.
 * 
 * @author oysteine
 * @version 1.0
 *
 */
public class PingBehave extends OneShotBehaviour {
  protected static final long serialVersionUID = 9200100880741959482L;
  protected Landmark myTarget;
  protected AgentTemplate myAgentTemplate;
  protected MinRTT myMinRTT;
  protected int myNumPings = 5;
  protected int myPingDistance = 3;
  protected boolean estimateEps = false;
  protected boolean estimateC1 = false;

  /**
   * Uses default values for probeCount and interprobe delay
   * No estimation of epsilon or C1, no rePing()
   * 
   * @param a the AgentTemplate this instance belongs to
   * @param landmark the target to ping
   */
  public PingBehave(AgentTemplate a, Landmark target) {
    super(a);
    myAgentTemplate = a;
    myTarget = target;
  }

  /**
   * Uses supplied values
   * TODO currently only IPv4
   * 
   * @param a the AgentTemplate this instance belongs to
   * @param landmark the target to ping
   * @param numPings the number of pings to send
   * @param pingDistance the time between pings are sent
   * @param estimateEps true if epsilon is to be estimated
   * @param estimateC1 true if C1 is to be estimated
   */
  public PingBehave(AgentTemplate a, Landmark target, int numPings, 
      int pingDistance, boolean estimateEps, boolean estimateC1){
    super(a);
    myAgentTemplate = a;
    myTarget = target;
    myNumPings = numPings;
    myPingDistance = pingDistance;
    this.estimateEps = estimateEps;
    this.estimateC1 = estimateC1;
  }

  /* (non-Javadoc)
   * @see jade.core.behaviours.Behaviour#action()
   */
  @Override
  public void action() {
    myMinRTT = new MinRTT();
    try {
      myMinRTT.ping(myTarget.getIP(), myNumPings, myPingDistance, false);
      if(estimateC1){
        myMinRTT.computeCRegions(true);
        myTarget.setC1(myMinRTT.getCI());
      }
      if(estimateEps){
        myTarget.setEpsilon(myMinRTT.estimateEpsilon());
      }
      myTarget.setMinRTT(myMinRTT.getMinRTT());

    } catch (HostUnreachableException e) {
    }
  }

}//class