package kripos.geo;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;

import jade.content.ContentElement;
import jade.content.lang.Codec.CodecException;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.OntologyException;
import jade.content.onto.UngroundedException;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import kripos.gateway.CommandPackage;
import kripos.ontology.*;

/**
 * Used by SessionAgent when a WorkerAgent is finished and reports in
 * 
 * @author oysteine
 * @version 1.0
 */
public class TraceReduceBehaviour extends OneShotBehaviour{
  private static final long serialVersionUID = -7988728778910969955L;
  private SessionAgent mySessionAgent;
  private ACLMessage myMsg;
  private AID myWorkerAID;
  private IP myTarget;
  private String myType = null;
  private String myWorkerDoneStatus = "OK";

  /**
   * Creates an instance of this Behaviour.
   * 
   * @param a the SessionAgent that owns this instance
   * @param m the ACLMessage this behaviour will work on
   */
  public TraceReduceBehaviour(SessionAgent a, ACLMessage m) {
    super(a);
    mySessionAgent = a;
    myMsg = m;
  }

  /**
   * Utility method for constructing LogRecs and logging them.
   * 
   * @param logContent The content of the LogRec
   * @param logType The Type of the LogRec
   * @param caseID The CaseID of the LogRec
   */
  private void log(String logContent, String logType, int caseID){
    LogRec logr = new LogRec();
    logr.setHasCaseID(caseID);
    logr.setHasSessionID(mySessionAgent.getSession().getSessionID());
    logr.setLogType(logType);
    OntDate od = new OntDate();
    od.setTime(mySessionAgent.getCalendar().getTimeInMillis());
    logr.setHasDate(od);
    logr.setLogLevel("INFO");
    logr.setLogContent(logContent);
    mySessionAgent.getLogger().log(logr);
  }

  /* (non-Javadoc)
   * @see jade.core.behaviours.Behaviour#action()
   */
  @Override
  public void action() {
    //TODO update db?
    try {
      myWorkerAID = myMsg.getSender();
      ContentElement ce = mySessionAgent.getContentManager().extractContent(myMsg);
      TraceResultList trl = (TraceResultList)((Action)ce).getAction();
      ArrayList<TraceResult> traceResults = new ArrayList<TraceResult>();
      Iterator it = trl.getAllTraceResult();
      while(it.hasNext()){
        traceResults.add((TraceResult)it.next());
      }
      myTarget = (IP)(traceResults.get(0)).getAddressToBeTraced();

      if(traceResults.size()<2){//TODO check type instead!!
        //TODO check for error
        myType = "TRACE-CBG";
        trl.getDoneStatus(); //TODO set in total and discard				
        (mySessionAgent.getAccTraceResults()).addResult(traceResults.get(0));
      }
      else {
        myType = "TRACE-GeoPing";
        ArrayList<DelayVector> dVectors = mySessionAgent.getAccTraceResults().getDelayVectors();
        if(dVectors.isEmpty()){
          ArrayList<String> names = landmarkNames(myTarget);
          for(int i=0;i<names.size();i++){
            DelayVector dv = new DelayVector(names.get(i));
            dVectors.add(dv);
          }
        }
        for(int i=0;i<dVectors.size();i++){
          for(int j=1;j<traceResults.size();j++){//skip the TraceResult to target
            if(dVectors.get(i).getName().equalsIgnoreCase(((DNSname)(traceResults.get(j).getAddressToBeTraced())).getHasName())){
              dVectors.get(i).add(traceResults.remove(j).getTraceResultData(), traceResults.get(0).getTraceResultData());
            }
          }
        }
      }

      //TODO myWorkerDoneStatus = done.getDoneStatus(); //fix tilsvarende i trace
      //TODO mySessionAgent.setDoneStatus(myWorkerDoneStatus);

      if(myWorkerDoneStatus.equalsIgnoreCase("OK")){
        reduce();

        log("WorkerAgent:" + myWorkerAID + 
            "finished successfully, terminated", "Type:Session", 
            mySessionAgent.getSession().getCaseID());				
      }
      else if(myWorkerDoneStatus.equalsIgnoreCase("Fail")){
        //Should do something more. Why failed? Create new worker?
        reduce();

        log("WorkerAgent:" + myWorkerAID + "failed due to" +//FIXME done.getHasReason() +
            ", terminated", "Type:Session", 
            mySessionAgent.getSession().getCaseID());				
      }
      else{
        ACLMessage reply = myMsg.createReply();
        reply.setPerformative(ACLMessage.NOT_UNDERSTOOD);
        myAgent.send(reply);
      }

    } catch (UngroundedException e) {
      e.printStackTrace();
    } catch (CodecException e) {
      e.printStackTrace();
    } catch (OntologyException e) {
      e.printStackTrace();
    }
  }//action()

  /**
   * Cleans up after a WorkerAgent is Done, and optionally terminates the entire Session.
   * <p>
   * Removes the workerAgent that sent the TraceList from myWorkerAgents.
   * Sets the doneStatus of the sessionAgent to the doneStatus in the incoming Done.
   * Checks if there are any more WorkerAgents, if not terminate the session and kill
   * the owning SessionAgent after informing its AdminAgent.
   */
  private void reduce(){
    mySessionAgent.removeWorker(myWorkerAID);
    //TODO mySessionAgent.setDoneStatus(myWorkerDoneStatus);

    if (mySessionAgent.getWorkers().isEmpty()){
      TraceResultList trl = new TraceResultList();
      CommandPackage cp = mySessionAgent.getAccTraceResults();
      if(myType.equalsIgnoreCase("Trace-CBG")){
        ArrayList<TraceResult> results = cp.getCBGResults();
        for(int i=0;i<results.size();i++){
          trl.addTraceResult(results.get(i));
        }

      }
      else if(myType.equalsIgnoreCase("Trace-GeoPing")){
        ArrayList<DelayVector> dList = cp.getDelayVectors();
        ArrayList<Object[]> lInfo = landmarkInfo(myTarget);

        double one = 99999999;//arbitrary high values
        double two = 99999999;
        double three = 99999999;
        double four = 99999999;
        TraceResult tr1 = new TraceResult();
        TraceResult tr2 = new TraceResult();
        TraceResult tr3 = new TraceResult();
        TraceResult tr4 = new TraceResult();

        for (int i=0;i<dList.size();i++){
          Double temp = dList.get(i).euclidianDistance();
          if(temp<one && temp>0){
            one = temp;
            for(int j=0;j<lInfo.size();j++){
              if(dList.get(i).getName().equalsIgnoreCase((String)lInfo.get(j)[0])){
                GeoLocation geo = new GeoLocation();
                geo.setLocationName((String)lInfo.get(j)[0]);
                geo.setLocationLatitude(((Double)lInfo.get(j)[1]).floatValue());
                geo.setLocationLongitude(((Double)lInfo.get(j)[2]).floatValue());
                tr1.setHasGeoLocation(geo);
              }
            }
            tr1.setTraceResultData(temp.floatValue());
          }
          else if(temp<two && temp>0){
            two = temp;
            for(int j=0;j<lInfo.size();j++){
              if(dList.get(i).getName().equalsIgnoreCase((String)lInfo.get(j)[0])){
                GeoLocation geo = new GeoLocation();
                geo.setLocationName((String)lInfo.get(j)[0]);
                geo.setLocationLatitude(((Double)lInfo.get(j)[1]).floatValue());
                geo.setLocationLongitude(((Double)lInfo.get(j)[2]).floatValue());
                tr2.setHasGeoLocation(geo);
              }
            }
            tr2.setTraceResultData(temp.floatValue());
          }

          else if(temp<three && temp>0){
            three = temp;
            for(int j=0;j<lInfo.size();j++){
              if(dList.get(i).getName().equalsIgnoreCase((String)lInfo.get(j)[0])){
                GeoLocation geo = new GeoLocation();
                geo.setLocationName((String)lInfo.get(j)[0]);
                geo.setLocationLatitude(((Double)lInfo.get(j)[1]).floatValue());
                geo.setLocationLongitude(((Double)lInfo.get(j)[2]).floatValue());
                tr3.setHasGeoLocation(geo);
              }
            }
            tr3.setTraceResultData(temp.floatValue());
          }

          else if(temp<four && temp>0){
            four = temp;
            for(int j=0;j<lInfo.size();j++){
              if(dList.get(i).getName().equalsIgnoreCase((String)lInfo.get(j)[0])){
                GeoLocation geo = new GeoLocation();
                geo.setLocationName((String)lInfo.get(j)[0]);
                geo.setLocationLatitude(((Double)lInfo.get(j)[1]).floatValue());
                geo.setLocationLongitude(((Double)lInfo.get(j)[2]).floatValue());
                tr4.setHasGeoLocation(geo);
              }
            }
            tr4.setTraceResultData(temp.floatValue());
          }

        }
        trl.addTraceResult(tr1);
        trl.addTraceResult(tr2);
        trl.addTraceResult(tr3);
        trl.addTraceResult(tr4);
      }

      trl.setDoneStatus(mySessionAgent.getDoneStatus());			
      ACLMessage isDone = new ACLMessage(ACLMessage.INFORM);
      isDone.addReceiver(mySessionAgent.getAdmin());
      isDone.setLanguage(new SLCodec(0).getName());
      isDone.setOntology(InternetInvestigationsOntology.getInstance().getName());
      String convID = mySessionAgent.getConvList().registerConversation();
      isDone.setConversationId(convID);			
      Action act = new Action(mySessionAgent.getAdmin(), trl);

      try {
        mySessionAgent.getContentManager().fillContent(isDone, act);
        mySessionAgent.send(isDone);
      } catch (CodecException e) {
        e.printStackTrace();
      } catch (OntologyException e) {
        e.printStackTrace();
      }

      ACLMessage doneReply = mySessionAgent.blockingReceive(MessageTemplate.MatchConversationId(convID), 90000);
      if (doneReply != null){
        if (doneReply.getPerformative() == ACLMessage.INFORM);
        /* Log that we have reported done and terminated after receiving reply*/
        log(myAgent.getName() + "finished", "Type:Session", 
            mySessionAgent.getSession().getCaseID());

        myAgent.doDelete();
      }
      else{
        /* Log that we have reported done and terminated on timeout*/
        log(myAgent.getName() + "finished" + "selfterminated",
            "Type:Session", mySessionAgent.getSession().getCaseID());				
        myAgent.doDelete();
      }
    }//outer if


  }//reduce()

  /**
   * Get the names of landmarks from the Landmarks database, 
   * excluding any landmark matching the input IP adress.
   * 
   * @param IP address to exclude
   * @return the number of landmarks
   */
  private ArrayList<String> landmarkNames(IP ip){
    Connection con = connectDB();
    ArrayList<String> names = new ArrayList<String>();
    try {
      Statement stmt = con.createStatement();
      String query = "SELECT NAME FROM LANDMARKS WHERE IPADR <>'"+ip.getHasName()+"'";
      ResultSet rs = stmt.executeQuery(query);
      while(rs.next()){
        names.add(rs.getString(1));
      }
    }
    catch(SQLException se){
      se.printStackTrace();
      return names;
    }
    return names;
  }

  /**
   * Get the names and geographic locations of landmarks from the Landmarks database, 
   * excluding any landmark matching the input IP adress.
   * 
   * @param IP address to exclude
   * @return an array containing the name and latitude and longitude
   */
  private ArrayList<Object[]> landmarkInfo(IP ip){
    Connection con = connectDB();
    ArrayList<Object[]> info = new ArrayList<Object[]>();
    try {
      Statement stmt = con.createStatement();
      String query = "SELECT NAME, LATITUDE, LONGITUDE FROM LANDMARKS WHERE IPADR <>'"+ip.getHasName()+"'";
      ResultSet rs = stmt.executeQuery(query);
      while(rs.next()){
        Object[] element = new Object[3];
        element[0] = rs.getString("NAME");
        element[1] = new Double(rs.getDouble("LATITUDE"));
        element[2] = new Double(rs.getDouble("LONGITUDE"));
        info.add(element);
      }
    }
    catch(SQLException se){
      se.printStackTrace();
      return info;
    }
    return info;
  }

  /**
   * Connects to the local database and returns the Connection for further use.
   * 
   * @return a connection to the database
   */
  private Connection connectDB(){//TODO path & password
    Connection c = null;
    try {
      Class.forName("org.hsqldb.jdbcDriver");
      c = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "sa", "");
    }catch (ClassNotFoundException e) {
      //FIXME unable to find database classes. dosomething
      e.printStackTrace();
    } 
    catch (SQLException e) {
      //FIXME unable to connect to database dosomething
      e.printStackTrace();
    }
    return c;
  }

}//class
