package kripos.geo;

import ie.moguntia.webcrawler.SaveURL;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;
import jade.content.lang.Codec.CodecException;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import kripos.ontology.*;

/**
 * THIS IS A RUDIMENTARY TEST CLASS
 * <p> 
 * It does NOT correctly mirror the content of a website!
 * In particular no embedded objects (like images) are downloaded, and depth is 1.
 * It also does not recreate the directory hierarchy of the remote site, 
 * but saves all downloaded files in the default directory of the JVM!
 * <p>
 * It also does not use its ontological input, only a string representing an url.
 * <p>
 * It is meant SOLELY for testing and demonstration purposes.
 * 
 * @author oysteine
 * @version 1.0
 */
public class WorkerCrawlBehaviour extends OneShotBehaviour {
   private static final long serialVersionUID = -2983783098953818393L;
   private WorkerAgent myWorkerAgent;
   private OntAddress myAddress;
   private Path myPath;
   private String myUrlString;
   private Vector myUrls;

   /**
    * Create a new instance
    * 
    * @param a the WorkerAgent this instance belongs to
    * @param ad the OntAddress this behaviour is to work with
    * @param p the Path this behaviour is to work with
    * @param url the url this behaviour is to crawl 
    */
   public WorkerCrawlBehaviour(WorkerAgent a, OntAddress ad, Path p, String url) {
      super(a);
      myWorkerAgent = a;
      myAddress = ad;
      if (p != null){
         myPath = p;
      }
      else{
         Directory d = new Directory();
         d.setHasName("/");
         myPath = new Path();
         myPath.addHasDirectoryList(d);
      }
      if (url != null){
         myUrlString = url;
      }
   }

   /**
    * Does the actual crawling. Downloads and crawls the file represented by 
    * the base-url, then downloads all links contained in this file.
    * <p>
    * All downloaded files are cheksummed.
    * <p>
    * Sends a Done message to the SessionAgent of its owner when its finished, 
    * and terminates.
    *  
    * @see jade.core.behaviours.Behaviour#action()
    */
   @Override
   public void action() {
      OntDone done = new OntDone();
      done.setDoneStatus("OK");
      try {
         URL url = new URL(myUrlString);
         myUrls = SaveURL.extractLinks(url);
         String noEscape = myUrlString.replace("://", "");  // illegal filename :// 
         noEscape = noEscape.replace("/", "-"); // illegal filename / 
         CheckSumGenerator csg = new CheckSumGenerator();
         //save the page represented by the base URL
         SaveURL.writeURLtoFile(url, noEscape);
         String startChkSum = csg.calcChkSum(noEscape);

         //writes the cheksum to file //should log this?
         FileWriter startWriter = new FileWriter(noEscape + "-chksum");
         startWriter.write(startChkSum);
         startWriter.close();

         log("Downloaded " + myUrlString + " Saved it to: "+ noEscape + "with checksum: " 
               + startChkSum, "Type:Session");

         //saves all pages linked to from the base URL and calculates the individual checksums
         for (int i=0; i< myUrls.size(); i++){
            String urlStr = (String)myUrls.get(i);
            String loopNoEscape = urlStr.replace("://", "");  // illegal filename ://
            loopNoEscape = loopNoEscape.replace("/", "-");  // illegal filename /
            URL loopUrl = new URL(urlStr);
            SaveURL.writeURLtoFile(loopUrl, loopNoEscape);

            String chkSum = csg.calcChkSum(loopNoEscape);

            //writes the cheksum to file //should log this?
            FileWriter writer = new FileWriter(loopNoEscape + "-chksum");
            writer.write(chkSum);
            writer.close();
            /* Logs the action along with checksum value */
            log("Downloaded " + urlStr + " Saved it to: "+ loopNoEscape + "with checksum: " 
                  + chkSum, "Type:Session");
         }
      } catch (MalformedURLException e) {
         done.setDoneStatus("Fail");
         // e.printStackTrace(); //generates too much noise
      } catch (IOException e) {
         done.setDoneStatus("Fail");
         // e.printStackTrace(); //generates too much noise
      }

      /* report to SessionAgent that we are finished*/
      ACLMessage isDone = new ACLMessage(ACLMessage.INFORM);
      isDone.addReceiver(myWorkerAgent.getSessionAgent());
      isDone.setLanguage(new SLCodec(0).getName());
      isDone.setOntology(InternetInvestigationsOntology.getInstance().getName());

      Action act = new Action(myWorkerAgent.getSessionAgent(), done);
      String convID = myWorkerAgent.getConvList().registerConversation();
      isDone.setConversationId(convID);

      try {
         myAgent.getContentManager().fillContent(isDone, act);
         myWorkerAgent.send(isDone);
         myAgent.doDelete();

      } catch (CodecException e) {
         System.out.println("Sending of ACL message failed");
         e.printStackTrace();
      } catch (OntologyException e) {
         System.out.println("Sending of ACL message failed");
         e.printStackTrace();
      }

   }//action()

   /**
    * Utility method for constructing LogRecs and logging them.
    * 
    * @param logContent The content of the LogRec
    * @param logType The Type of the LogRec
    * @param caseID The CaseID of the LogRec
    */
   private void log(String logContent, String logType){
      LogRec logr = new LogRec();
      logr.setHasCaseID(myWorkerAgent.getCaseID());
      logr.setHasSessionID(myWorkerAgent.getSessionID());
      logr.setLogType(logType);
      OntDate od = new OntDate();
      od.setTime(myWorkerAgent.getCalendar().getTimeInMillis());
      logr.setHasDate(od);
      logr.setLogLevel("INFO");
      logr.setLogContent(logContent);
      myWorkerAgent.getLogger().log(logr);
   }

}//class