package kripos.geo;

import jade.content.Concept;
import kripos.ontology.*;
import jade.content.ContentElement;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.UngroundedException;
import jade.content.onto.basic.Action;
import jade.core.behaviours.CyclicBehaviour;
import jade.lang.acl.*;

/**
 * Behaviour that handles incoming messages for {@link kripos.AdminAgent}s.
 * Does not handle replies, only new messages.
 * 
 * @author oysteine
 * @version 1.0
 */
public class WorkerReceiveBehaviour extends CyclicBehaviour {

   private static final long serialVersionUID = -2757785238801332813L;
   private WorkerAgent myWorkerAgent;	

   /**
    * Creates an instance of this behaviour
    * @param a the agent this instance belongs to 
    */
   public WorkerReceiveBehaviour(WorkerAgent a){
      super(a);
      myWorkerAgent = a;
   }

   /**
    * Matches any message that is not sent in reply to another message.
    * Replies are handled by other behaviours. 
    */
   public @Override void action() {        
      MessageTemplate mt = myWorkerAgent.getConvList().getMessageTemplate();
      ACLMessage msg = myAgent.receive(mt);
      if(msg != null){
         handle(msg);
      }
      block(150);
   }

   /**
    * Handles incoming messages, and dispatches them to an appropriate behaviour
    * 
    * @param msg incoming {@link jade.lang.acl.ACLMessage}
    */
   private void handle(ACLMessage msg){
      try {
         switch (msg.getPerformative()) {     
         case (ACLMessage.REQUEST):
            ContentElement content = myAgent.getContentManager().extractContent(msg);
         Concept action = ((Action)content).getAction();

         if (action instanceof Report){
            myAgent.addBehaviour(new WorkerReportBehaviour(myWorkerAgent, msg));
         }
         else if (action instanceof Abort){
            //todo log to syslog
            myAgent.doDelete();
         }
         else if (action instanceof UpdateTime){
            UpdateTime ut = (UpdateTime)action;
            long toAdd = (long)ut.getHasOffset();
            myWorkerAgent.getCalendar().setTimeInMillis(
                  myWorkerAgent.getCalendar().getTimeInMillis() + toAdd);
         }

         else{
            ACLMessage reply = msg.createReply();
            reply.setPerformative(ACLMessage.NOT_UNDERSTOOD);
            myAgent.send(reply);
         }
         break;
         } 
      }
      catch (UngroundedException e) {
         e.printStackTrace();
      } 

      catch (CodecException e) {
         e.printStackTrace();
      } 

      catch (OntologyException e) {
         e.printStackTrace();
      }
   }//handle()

}//class