package kripos.geo;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import Jama.Matrix;
import jade.core.behaviours.OneShotBehaviour;
import jade.core.behaviours.SequentialBehaviour;
import kripos.ontology.DNSname;
import kripos.ontology.IP;

public class WorkerTraceCalculate extends OneShotBehaviour {
  private WorkerAgent myWorkerAgent;
  private Landmark toTrace;
  private double[] myBestLine = new double[2];
  private ArrayList<Landmark> myLandmarks;
  private double baseM = 0.02; //magic number due to lightspeed in optical fibres
  private long allowedSlack =0;

  /**
   * 
   * @param a
   * @param toTrace
   */
  public WorkerTraceCalculate(WorkerAgent a, Landmark toTrace) {
    super(a);
    myWorkerAgent = a;
    this.toTrace = toTrace;
  }

  @Override
  public void action() {
    myLandmarks = myWorkerAgent.getPingList().getLandmarkList();

    for(int o=0; o<myLandmarks.size();o++){
      Landmark l = myLandmarks.get(o);
    }

    if(myWorkerAgent.getType().equalsIgnoreCase("TRACE-CBG")){
      myBestLine = calculateBestline(myLandmarks);
    }

    Connection con = connectDB();
    updateDB(con, myBestLine, myLandmarks);

    ArrayList<String> ips =  newlyTraced(con);

    if (ips.get(ips.size()-1).equalsIgnoreCase("true")){
      toTrace = new Landmark(ips.get(0));
      Statement st;
      try {
        st = con.createStatement();
        String query2 = "SELECT NAME, MIN_RTT, C1, EPSILON, IPADR, CHECKED FROM TRACED WHERE IPADR = '"+ips.get(0)+"'";
        ResultSet rs2 = st.executeQuery(query2);
        while(rs2.next()){
          double minRtt = rs2.getDouble("MIN_RTT");
          double c1 = rs2.getDouble("C1");
          double epsilon = rs2.getDouble("EPSILON");
          toTrace.setMinRTT(minRtt);
          toTrace.setC1(c1);
          toTrace.setEpsilon(epsilon);
        }
        ((SequentialBehaviour)root()).addSubBehaviour(
            new WorkerTraceFinal(myWorkerAgent, toTrace, myBestLine, myLandmarks));

      } catch (SQLException e) {
        e.printStackTrace();
      }
    }
    else{
      ((SequentialBehaviour)root()).addSubBehaviour(
          new WorkerTraceTarget(myWorkerAgent, toTrace, myBestLine, myLandmarks));
    }
  }

  /**
   * Calculates the bestline from ping measurements
   * 
   * @param landmarks the landmarks used as a basis for calculating the bestline
   * @return an array containing the two double values that constitutes the bestline
   */
  private double[] calculateBestline(ArrayList<Landmark> landmarks){
    double[] result = new double[2];
    double bestMi = 9999999;//initial nonsense values
    double bestBi = 9999999;

    double smallestY = 99999999;
    double smallestX = 0;
    int startAt = 1;  //requires sorted input!
    for(int i=0;i<landmarks.size();i++){
      double tempY = landmarks.get(i).getMinRTT();
      //set new value for smallestY, but only if it is sane
      if(tempY < smallestY && landmarks.get(i).getMinRTT() >= landmarks.get(i).getDistance() * baseM){
        smallestY = tempY;
        startAt = i+1;//only use landmarks with greater distance for calculating bestline
        smallestX = landmarks.get(i).getDistance();
      }
    }
    //set up matrices for solving y = mx + b
    Matrix a = new Matrix(2,2);
    Matrix b = new Matrix(2,1);
    //fill matrices with values from landmark containing smallestY
    a.set(0, 0, smallestX);
    a.set(0, 1, 1);
    b.set(0, 0, smallestY);

    //solve y = mx + b for all the pairs (smallestY, other landmark)
    for(int i=startAt;i<landmarks.size();i++){
      a.set(1, 0, landmarks.get(i).getDistance());
      a.set(1, 1, 1);

      b.set(1, 0, landmarks.get(i).getMinRTT());

      Matrix x = a.solve(b);
      if((x.get(0,0)>baseM) && (x.get(1,0)>=0)){
        if(x.get(0,0)< bestMi){
          bestMi = x.get(0,0);
          bestBi = x.get(1,0);
        }
      }
    }

    //choose next-smallest Y if result does not fall within allowed region
    if(bestMi<baseM || bestBi<0){
      if(landmarks.size()>4){
        landmarks.remove(startAt-1);//remove the outlier measurement
        calculateBestline(landmarks);
      }
      result[0] = bestMi;
      result[1] = bestBi;
      return result;	//TODO throw exception instead	
    }
    else{
      result[0] = bestMi;
      result[1] = bestBi;
      return result;
    }
  }

  /**
   * Connects to the local database and returns the Connection for further use.
   * 
   * @return a connection to the database
   */
  private Connection connectDB(){//TODO path & password
    Connection c = null;
    try {
      Class.forName("org.hsqldb.jdbcDriver");
      c = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "sa", "");
    }catch (ClassNotFoundException e) {
      //FIXME unable to find database classes. dosomething
      e.printStackTrace();
    } 
    catch (SQLException e) {
      //FIXME unable to connect to database dosomething
      e.printStackTrace();
    }
    return c;
  }

  /**
   * Checks if an IP address has currently been traced.
   * The definition of current is given by allowedSlack.
   * 
   * @param con The database connection to use
   * @return true if this IP was recently traced
   */
  private ArrayList<String> newlyTraced(Connection con){
    String newlyTraced = "false";
    ArrayList<String> returnList = new ArrayList<String>();
    try {
      String adrToCheck = toTrace.getIP();
      Statement stmt = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_READ_ONLY);
      String query = "SELECT NAME, IPADR, CHECKED FROM TRACED WHERE IPADR = '"+adrToCheck+"'";
      ResultSet rs = stmt.executeQuery(query);
      if(!rs.next()){
        returnList.add(adrToCheck);
      }
      rs.previous();
      while (rs.next()) {
        Date lastChecked  = (Date)rs.getTimestamp("CHECKED");
        long toCheck;
        if(rs.wasNull()){
          toCheck = 0;
        }
        else{
          toCheck = lastChecked.getTime();
        }
        long currentTime = myWorkerAgent.getCalendar().getTimeInMillis();
        long diff = currentTime - toCheck;
        if(diff<allowedSlack){
          returnList.add(rs.getString("IPADR"));
          newlyTraced = "true";
        }
        else{
          returnList.add(adrToCheck);
        }
      }
    } 
    catch (SQLException e) {
      e.printStackTrace();
      returnList.add(newlyTraced);
      return returnList;
    }
    returnList.add(newlyTraced);
    return returnList;
  }	

  /**
   * Updates the MISC table with the new bestline information and 
   * any matching landmarks in the LANDMARK database with new minRTT, C1, Epsilon and timestamp
   *
   * @param con The Connection to use
   * @param mi the m part of the new bestline
   * @param bi the b part of the new bestline
   */
  private void updateDB(Connection con, double [] bestline, ArrayList<Landmark> newLandmarks){
    String newBestline = "UPDATE MISC SET LAST_BESTLINE = now ,BESTLINE_M = "+
    bestline[0]+", BESTLINE_B ="+ bestline[1];

    try {
      Statement st = con.createStatement();
      st.executeUpdate(newBestline);
      for(int i=0; i<newLandmarks.size();i++){
        String landmarkUpdate = "UPDATE LANDMARKS SET MIN_RTT= "
          +newLandmarks.get(i).getMinRTT()+", CHECKED = now, C1 = "+newLandmarks.get(i).getC1()+
          ", EPSILON = "+newLandmarks.get(i).getEpsilon();
        st.executeUpdate(landmarkUpdate);
      }
    } catch (SQLException e) {
      e.printStackTrace();
    }
  }

}//class
