package kripos.geo;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import jade.content.lang.Codec.CodecException;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import kripos.ontology.DNSname;
import kripos.ontology.GeoLocation;
import kripos.ontology.IP;
import kripos.ontology.InternetInvestigationsOntology;
import kripos.ontology.TraceResult;
import kripos.ontology.TraceResultList;

public class WorkerTraceFinal extends OneShotBehaviour {
  private static final long serialVersionUID = -6054469352616818627L;
  private WorkerAgent myWorkerAgent;
  private Landmark toTrace;
  private double[] myBestLine;
  private ArrayList<Landmark> myLandmarks;


  public WorkerTraceFinal(WorkerAgent a, Landmark toTrace, double[] bestLine, 
      ArrayList<Landmark> landmarks){
    super(a);
    myWorkerAgent = a;
    this.toTrace = toTrace;
    myBestLine = bestLine;
    myLandmarks = landmarks;
  }

  @Override
  public void action() {
    ACLMessage returnResult = new ACLMessage(ACLMessage.INFORM);
    returnResult.addReceiver(myWorkerAgent.getSessionAgent());
    returnResult.setLanguage(new SLCodec(0).getName());
    returnResult.setOntology(InternetInvestigationsOntology.getInstance().getName());
    String convID = myWorkerAgent.getConvList().registerConversation();
    returnResult.setConversationId(convID);
    Action act;
    IP target = new IP();
    target.setHasName(toTrace.getName());
    GeoLocation gl = new GeoLocation();

    try{
      Connection con = connectDB();
      Statement st = con.createStatement();
      String getGeoLocation = "SELECT NAME, LATITUDE, LONGITUDE FROM MISC";
      ResultSet rs = st.executeQuery(getGeoLocation);
      while(rs.next()){
        Double lat = rs.getDouble("LATITUDE");
        Double lon = rs.getDouble("LONGITUDE");
        String name = rs.getString("NAME");
        gl.setLocationName(name);
        gl.setLocationLatitude(lat.floatValue());
        gl.setLocationLongitude(lon.floatValue());
      }
    }
    catch(SQLException se){
      se.printStackTrace();
    }

    if(myWorkerAgent.getType().equalsIgnoreCase("TRACE-CBG")){
      TraceResultList trl = new TraceResultList();
      trl.setHasGeoLocation(gl);
      TraceResult tr = new TraceResult();
      tr.setAddressToBeTraced(target);//TODO only first IP address if DNSName 
      tr.setHasCaseID(myWorkerAgent.getCaseID());
      tr.setHasSessionID(myWorkerAgent.getSessionID());
      tr.setHasGeoLocation(gl);

      Double geoDistance = (toTrace.getMinRTT() - myBestLine[1])/myBestLine[0];
      tr.setTraceResultData(geoDistance.floatValue());

      trl.addTraceResult(tr);
      act = new Action(myWorkerAgent.getSessionAgent(), trl);
    }

    else if(myWorkerAgent.getType().equalsIgnoreCase("TRACE-GeoPing")){
      TraceResultList trl = new TraceResultList();
      trl.setHasGeoLocation(gl);

      TraceResult targetTr = new TraceResult();
      targetTr.setTraceResultData((new Double(toTrace.getMinRTT()).floatValue()));
      targetTr.setAddressToBeTraced(target);
      trl.addTraceResult(targetTr);

      for (int i=0;i<myLandmarks.size();i++){
        TraceResult tr = new TraceResult();
        DNSname name = new DNSname();
        name.setHasName(myLandmarks.get(i).getName());
        tr.setAddressToBeTraced(name);
        tr.setTraceResultData((new Double(myLandmarks.get(i).getMinRTT())).floatValue());
        trl.addTraceResult(tr);
      }

      act = new Action(myWorkerAgent.getSessionAgent(), trl);

    }
    else{
      TraceResultList trl = new TraceResultList();
      trl.setDoneStatus("failed");
      act = new Action(myWorkerAgent.getSessionAgent(), trl);
    }

    try {
      myAgent.getContentManager().fillContent(returnResult, act);
      myWorkerAgent.send(returnResult);
      myAgent.doDelete();

    } catch (CodecException e) {
      System.out.println("Sending of ACL message failed");
      e.printStackTrace();
    } catch (OntologyException e) {
      System.out.println("Sending of ACL message failed");
      e.printStackTrace();
    }
  }

  /**
   * Connects to the local database and returns the Connection for further use.
   * 
   * @return a connection to the database
   */
  private Connection connectDB(){//TODO path & password
    Connection c = null;
    try {
      Class.forName("org.hsqldb.jdbcDriver");
      c = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "sa", "");
    }catch (ClassNotFoundException e) {
      //FIXME unable to find database classes. dosomething
      e.printStackTrace();
    } 
    catch (SQLException e) {
      //FIXME unable to connect to database dosomething
      e.printStackTrace();
    }
    return c;
  }

}//class