package kripos.geo;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import com.bbn.openmap.LatLonPoint;
import jade.core.behaviours.OneShotBehaviour;
import jade.core.behaviours.SequentialBehaviour;
import kripos.ontology.DNSname;
import kripos.ontology.IP;
import kripos.ontology.OntAddress;
import kripos.ontology.WorkItem;

public class WorkerTraceStart extends OneShotBehaviour {
  private static final long serialVersionUID = 824433708805797669L;
  private WorkerAgent myWorkerAgent;
  private WorkItem myWorkItem;
  private long allowedSlack = 0;//TODO set sane value

  public WorkerTraceStart(WorkerAgent a, WorkItem wi) {
    super(a);
    myWorkerAgent = a;
    myWorkItem = wi;
  }

  @Override
  public void action() {
    ArrayList<Landmark> landmarks = new ArrayList<Landmark>();
    double[] bestLine = new double[2]; 
    Landmark toTrace = null;
    try {
      toTrace = new Landmark(lookupIP().get(0));//TODO only uses first returned IP
    } catch (UnknownHostException e1) {
      e1.printStackTrace();
    }

    Connection con = connectDB();

    ArrayList<Landmark> initialLandmarks = readLandmarks(con);
    if(checkOldBestline(con)){
      landmarks = initialLandmarks;
      try {
        Statement stmt = con.createStatement();
        String query = "SELECT BESTLINE_M, BESTLINE_B FROM MISC";
        ResultSet rs = stmt.executeQuery(query);
        while(rs.next()){
          bestLine[0] = rs.getDouble("BESTLINE_M");
          bestLine[1] = rs.getDouble("BESTLINE_B");
        }

        ArrayList<String> ips =  newlyTraced(con);
        if (ips.get(ips.size()-1).equalsIgnoreCase("true")){
          toTrace = new Landmark(ips.get(0));
          try {
            Statement st = con.createStatement();
            String query2 = "SELECT NAME, MIN_RTT, C1, EPSILON, IPADR, CHECKED FROM TRACED WHERE IPADR = '"+ips.get(0)+"'";
            ResultSet rs2 = st.executeQuery(query2);
            while(rs2.next()){
              double minRtt = rs2.getDouble("MIN_RTT");
              double c1 = rs2.getDouble("C1");
              double epsilon = rs2.getDouble("EPSILON");
              toTrace.setMinRTT(minRtt);
              toTrace.setC1(c1);
              toTrace.setEpsilon(epsilon);
            }
            ((SequentialBehaviour)root()).addSubBehaviour(
                new WorkerTraceFinal(myWorkerAgent, toTrace, bestLine, landmarks));

          } catch (SQLException e) {
            e.printStackTrace();
          }
        }
        else {
          ((SequentialBehaviour)root()).addSubBehaviour(
              new WorkerTraceTarget(myWorkerAgent, toTrace, bestLine, landmarks));
        }
      }
      catch(SQLException se){
        //FIXME do something
        se.printStackTrace();
      }
    }
    else{
      WorkerTraceLandmarks part2 = new WorkerTraceLandmarks(myWorkerAgent, toTrace);
      myWorkerAgent.getPingList().setContent(initialLandmarks);
      for (int i=0; i<initialLandmarks.size();i++){
        PingBehave pb = new PingBehave(myWorkerAgent, initialLandmarks.get(i));
        part2.addSubBehaviour(part2.getTbf().wrap(pb));
      }

      ((SequentialBehaviour)root()).addSubBehaviour(part2);
    }

  }//action

  /**
   * If the class of myTarget is DNSName we need to get the IP address.
   * If the class of myTarget is IP no lookup is performed before returning the IP address.
   * 
   * @return An ArrayList of IP addresses as strings
   * @throws UnknownHostException 
   */
  private ArrayList<String> lookupIP() throws UnknownHostException {
    ArrayList<String> ipStrings = null;
    OntAddress oa = myWorkItem.getHasBaseAddress();
    if(oa instanceof DNSname) {
      DNSname dns = (DNSname)(oa);
      InetAddress[] ips;
      ips = InetAddress.getAllByName(dns.getHasName());
      ipStrings = new ArrayList<String>();
      for(int i=0;i<ips.length;i++){
        ipStrings.add(ips[i].getHostAddress());
      }
      return ipStrings;
    }
    else {
      ipStrings = new ArrayList<String>();
      ipStrings.add(((IP)myWorkItem.getHasBaseAddress()).getHasName());
      return ipStrings;
    }
  }

  /**
   * Reads landmarks from the local database.
   * 
   * @param con Database connection
   * @return a list of landmarsk sorted by increasing distance from the owner of this instance
   */
  private ArrayList<Landmark> readLandmarks(Connection con){
    ArrayList<Landmark> landmarks = new ArrayList<Landmark>();

    try {
      Statement stmt = con.createStatement();
      String query = "SELECT * FROM LANDMARKS ORDER BY DISTANCE_KM ASC";
      ResultSet rs = stmt.executeQuery(query);
      while(rs.next()){
        String name = rs.getString("NAME");
        String ipAdr = rs.getString("IPADR");
        double distance = rs.getDouble("DISTANCE_KM");
        LatLonPoint llp = new LatLonPoint(rs.getDouble("LATITUDE"), rs.getDouble("LONGITUDE"));

        Landmark l = new Landmark(name, llp, ipAdr);
        l.setDistance(distance);
        landmarks.add(l);
      }
    }catch (SQLException e) {
      // TODO: handle exception
    }
    return landmarks;
  }


  /**
   * Checks if an IP address has currently been traced.
   * The definition of current is given by allowedSlack.
   * 
   * @param con The database connection to use
   * @return true if this IP was recently traced
   */
  private ArrayList<String> newlyTraced(Connection con){
    String newlyTraced = "false";
    ArrayList<String> returnList = new ArrayList<String>();
    try {
      ArrayList<String> adrToCheck = lookupIP();
      Statement stmt = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_READ_ONLY);
      for(int i=0;i<adrToCheck.size();i++){
        String query = "SELECT NAME, IPADR, CHECKED FROM TRACED WHERE IPADR = '"+adrToCheck.get(i)+"'";
        ResultSet rs = stmt.executeQuery(query);
        if(!rs.next()){
          returnList.add(adrToCheck.get(i));
        }
        rs.previous();
        while (rs.next()) {
          Date lastChecked  = (Date)rs.getTimestamp("CHECKED");
          long toCheck;
          if(rs.wasNull()){
            toCheck = 0;
          }
          else{
            toCheck = lastChecked.getTime();
          }
          long currentTime = myWorkerAgent.getCalendar().getTimeInMillis();
          long diff = currentTime - toCheck;
          if(diff<allowedSlack){
            returnList.add(rs.getString("IPADR"));
            newlyTraced = "true";
          }
          else{
            returnList.add(adrToCheck.get(i));
          }
        }
      }
    } 
    catch (UnknownHostException uhe){
      uhe.printStackTrace();
      returnList.add("false");
      return returnList;
    }
    catch (SQLException e) {
      e.printStackTrace();
      returnList.add(newlyTraced);
      return returnList;
    }
    returnList.add(newlyTraced);
    return returnList;
  }

  /**
   * Check if the most recently calculated bestLine for this host is too old.
   * 
   * @return true if the current baseLine is usable
   */
  private boolean checkOldBestline(Connection con){
    Date lastChecked = new Date(0);
    try {
      Statement stmt = con.createStatement();
      String query = "SELECT LAST_BESTLINE FROM MISC";
      ResultSet rs = stmt.executeQuery(query);
      while (rs.next()) {
        if(rs.getTimestamp("LAST_BESTLINE") != null){
          lastChecked  = (Date)rs.getTimestamp("LAST_BESTLINE");
        }
      }

    } catch (SQLException e) {
      e.printStackTrace();
      return false;
    }

    long toCheck = lastChecked.getTime();
    long currentTime = myWorkerAgent.getCalendar().getTimeInMillis();
    long diff = currentTime - toCheck;

    if(diff <= allowedSlack){
      return true;
    }
    else{
      return false;
    }
  }

  /**
   * Connects to the local database and returns the Connection for further use.
   * 
   * @return a connection to the database
   */
  private Connection connectDB(){//TODO path & password
    Connection c = null;
    try {
      Class.forName("org.hsqldb.jdbcDriver");
      c = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "sa", "");
    }catch (ClassNotFoundException e) {
      //FIXME unable to find database classes. dosomething
      e.printStackTrace();
    } 
    catch (SQLException e) {
      //FIXME unable to connect to database dosomething
      e.printStackTrace();
    }
    return c;
  }

}//class