package kripos.geo;

import jade.core.behaviours.SequentialBehaviour;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

public class WorkerTraceTarget extends PingBehave {
  private static final long serialVersionUID = 1898594180371430543L;
  private WorkerAgent myWorkerAgent;
  private double[] myBestLine;
  private ArrayList<Landmark> myLandmarks;

  public WorkerTraceTarget(WorkerAgent a, Landmark target, double[] bestLine, 
      ArrayList<Landmark> landmarks) {
    super(a, target);
    myWorkerAgent = a;
    myBestLine = bestLine;
    myLandmarks = landmarks;
    myTarget = target;
  }

  public WorkerTraceTarget(WorkerAgent a, Landmark target, int numPings, 
      int pingDistance, boolean estimateEps, boolean estimateC1,  
      double[] bestLine, ArrayList<Landmark> landmarks){
    super(a, target, numPings, pingDistance, estimateEps, estimateC1);
    myWorkerAgent = a;
    myBestLine = bestLine;
    myLandmarks = landmarks;
    myTarget = target;
  }

  @Override
  public void action() {
    super.action();
    updateTracedDB(connectDB(), myTarget);
    ((SequentialBehaviour)root()).addSubBehaviour(
        new WorkerTraceFinal(myWorkerAgent, myTarget, myBestLine, myLandmarks));
  }

  /**
   * Creates a new entry in the TRACED database, 
   * or updates an existing entry if a mathcing IP address is found
   * 
   * @param con The database connection to use
   * @param toTrace the landmark containg the information to added
   */
  private void updateTracedDB(Connection con, Landmark toTrace) {
    try {
      Statement stmt = con.createStatement();
      String query = "SELECT COUNT(IPADR) FROM TRACED WHERE IPADR='"+toTrace.getIP()+"'";
      ResultSet rs = stmt.executeQuery(query);

      while(rs.next()){
        if(rs.getInt(1)>0){
          String updateString = "UPDATE TRACED SET MIN_RTT ="+ toTrace.getMinRTT()+", CHECKED= now WHERE IPADR='"+toTrace.getIP()+"'";
          stmt.executeUpdate(updateString);
        }
        else{
          String ip = toTrace.getIP();
          String name = toTrace.getName();
          double c1 = toTrace.getC1();
          double epsilon = toTrace.getEpsilon();
          double minRtt = toTrace.getMinRTT();
          String insertString = "INSERT INTO TRACED VALUES ('"+name+"','"+ip+"',now,"+minRtt+",null,"+
          c1+","+epsilon+",null)";
          stmt.executeUpdate(insertString);
        }
      }
    } catch (SQLException e) {
      e.printStackTrace();
    }
  }

  /**
   * Connects to the local database and returns the Connection for further use.
   * 
   * @return a connection to the database
   */
  private Connection connectDB(){//TODO path & password
    Connection c = null;
    try {
      Class.forName("org.hsqldb.jdbcDriver");
      c = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "sa", "");
    }catch (ClassNotFoundException e) {
      //FIXME unable to find database classes. dosomething
      e.printStackTrace();
    } 
    catch (SQLException e) {
      //FIXME unable to connect to database dosomething
      e.printStackTrace();
    }
    return c;
  }

}//class