package kripos.geo.openmap;

/**
 * Information about the intersection represented in a map.
 * 
 * @author oysteine
 */
public class IntersectionInfo {

  /**
   * Calculated area of intersection polygon.
   */
  public final double polygonArea;

  /**
   * Whether {@link #centroidLatitude} and {@link #centroidLongitude}
   * contains legal values.
   */
  public final boolean centroidAvailable;

  /**
   * Latitude (north of Equator) of polygon centroid.
   */
  public final double centroidLatitude;

  /**
   * Longitude (east of Greenwich) of polygon centroid.
   */
  public final double centroidLongitude;

  /**
   * Pixel size of image.
   */
  public final int imageWidth;

  /**
   * Pixel size of image.
   */
  public final int imageHeight;


  IntersectionInfo(double polygonArea, int imageWidth, int imageHeight) {
    this.polygonArea = polygonArea;
    this.centroidAvailable = false;
    this.centroidLatitude = Double.NaN;
    this.centroidLongitude = Double.NaN;

    this.imageWidth = imageWidth;
    this.imageHeight = imageHeight;
  }

  IntersectionInfo(double polygonArea, 
      double centroidLatitude, double centroidLongitude,
      int imageWidth, int imageHeight) {
    this.polygonArea = polygonArea;
    this.centroidAvailable = true;
    this.centroidLatitude = centroidLatitude;
    this.centroidLongitude = centroidLongitude;

    this.imageWidth = imageWidth;
    this.imageHeight = imageHeight;
  }

}//class