package kripos.geo.openmap;

import java.awt.AlphaComposite;
import java.awt.Composite;
import java.awt.Graphics;
import java.awt.Graphics2D;

import com.bbn.openmap.LatLonPoint;
import com.bbn.openmap.omGraphics.OMCircle;
import com.bbn.openmap.proj.Length;

/**
 * An {@link OMCircle} implementation that supports transparency.
 * 
 * Only the directly used constructors are implemented
 * 
 * @author oysteine
 */
public class OMAlphaCircle extends OMCircle implements Alpha {
  private static final long serialVersionUID = 9141255017478768485L; 
  private Composite composite;


  /**
   * Create an OMCircle with a lat/lon center and a physical
   * distance radius. Rendertype is RENDERTYPE_LATLON.
   * 
   * @param center LatLon center of circle
   * @param radius distance
   * @param units com.bbn.openmap.proj.Length object specifying
   *        units for distance.
   * @param nverts number of vertices for the poly-circle(if &lt; 3,
   *        value is generated internally)
   * @param alpha opacity in [0.0, 1.0]
   */
  public OMAlphaCircle(LatLonPoint center, float radius, Length units,
      int nverts, float alpha) {
    super(center, radius, units, nverts);
    composite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha);
  }


  /**
   * Set the alpha color
   * 
   * @param the color value to set
   */
  public void setAlpha(float alpha) {
    composite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha);
  }


  /**
   * Overriding the fill method to set alpha before filling and clearing it after.
   * @param g the <code>Graphics</code> instance to use
   */
  @Override
  public void fill(Graphics g) {
    Graphics2D g2 = (Graphics2D)g;
    Composite orig = g2.getComposite();
    g2.setComposite(composite);
    super.fill(g);
    g2.setComposite(orig);
  }

}//class
