package kripos.geo.openmap;

import java.awt.AlphaComposite;
import java.awt.Composite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;

import com.bbn.openmap.omGraphics.OMPoly;

/**
 * An {@link OMPoly} implementation that supports transparency.
 * 
 * Only the directly used constructors are implemented
 * 
 * @author oysteine
 */
public class OMAlphaPoly extends OMPoly implements Alpha {
  private static final long serialVersionUID = 597512041926004097L;
  private Composite composite;
  private Composite orig;


  /**
   * 
   * @param llPoints latitude, longitude, latitude, longitude, ...
   * @param units
   * @param lType
   * @param alpha opacity in [0.0, 1.0]
   */
  public OMAlphaPoly(float[] llPoints, int units, int lType, float alpha) {
    super(llPoints, units, lType);
    composite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha);
  }


  /**
   * Set the alpha color
   * 
   * @param the color value to set
   */
  public void setAlpha(float alpha) {
    composite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha);
  }


  @Override
  public void render(Graphics g) {
    // just to make sure we reset composite after
    Graphics2D g2 = (Graphics2D)g;
    orig = g2.getComposite();
    super.render(g);
    g2.setComposite(orig);
  }


  @Override
  public void setGraphicsForFill(Graphics g) {
    ((Graphics2D)g).setComposite(composite);
    super.setGraphicsForFill(g);
  }


  @Override
  public void setGraphicsColor(Graphics g, Paint paint) {
    ((Graphics2D)g).setComposite(composite);
    super.setGraphicsColor(g, paint);
  }

}//class