package kripos.geo.ping;
/*
 * Most of the functionality of this class is adapted from RTTOmeter.
 * While the main RTTOmeter application is released under the GPLv2, 
 * the file mode.c is apparently not (breach of GPL?). 
 * The notice below is required by the original author. 
 */ 

/*
 * Copyright (c) 2003, Amgad Zeitoun.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by Amgad Zeitoun at the University of Michigan, Ann Arbor. The
 * name of the University may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: 
 *           Amgad Zeitoun (azeitoun@eecs.umich.edu)
 */

import java.util.Hashtable;

public class ModeNode {
  private Long key;
  private long cnt;

  /**
   * 
   * @param x
   * @return
   */
  private int ceiling(float x){
    if( (int)(Math.abs(x) + 0.5) > (int)(Math.abs(x)) ){
      return (int)(Math.abs(x+0.5));
    }
    else {
      return (int)(Math.abs(x));
    }
  }

  /**
   * 
   * @param m1
   * @param m2
   * @return
   */
  public int matchNode(ModeNode m1, ModeNode m2){
    if(m1.getKey() < m2.getKey()) {
      return -1;
    }

    if(m1.getKey() > m2.getKey()) {
      return 1;
    }
    return 0;
  }


  /**
   * 	Calaculate the mode of an array of values with length len 
   * @param array
   * @param len
   * @return
   */
  public float mode(float[] array, int len){
    float mode = -1.0f;
    long max_cnt = 0;
    long val = 0;
    boolean dbl_precision;
    long key;

    if (array.length <= 0){
      return mode;
    }
    Hashtable<Long,ModeNode> hashtable = new Hashtable<Long,ModeNode>(array.length);

    /* If the RTTs are very small (i.e., less than 1 ms), make double digitis mode
		   precision, otherwise make it a single digit precision */
    /* NOTE: I assume that array is sorted. Which is true, because I call mode()
		   after I qsort the array
     */

    if (array[0] < 1.0){
      dbl_precision = true;
    }
    else{
      dbl_precision = false;
    }

    for (float element : array) {

      /* Using a single digit precision by default, except when the values
		    of RTTs are really small (<1.0ms) */
      if(dbl_precision){
        key = ceiling(element * 100);
      }
      else{
        key = ceiling(element * 10);
      }

      ModeNode m = hashtable.get(key);
      if(m == null) {
        m = new ModeNode();
        m.setKey(key);
        hashtable.put(m.getKey(), m);
      }

      m.incCnt();

      if(m.getCnt() > max_cnt) {
        /* we have a mode here */
        max_cnt = m.getCnt();
        val = key;
      }
      /* TODO: detect multiple modes */ //dette fikser vi
    }

    /* The mode should be repeated more than once! */
    /* Just in case we don't have any mode at all */
    if ( max_cnt > 1 ) {
      if(dbl_precision){
        mode = (float)val/100;
      }
      else{
        mode = (float)val/10;
      }
    }
    return mode;

  }//mode

  /*
   * Setter and getter methods
   */

  public void incCnt(){
    cnt++;
  }


  public long getCnt(){
    return cnt;
  }


  public Long getKey(){//param?
    return key;
  }


  public void setKey(Long newKey){
    key = newKey;
  }

}//class