//legacy code. only works in 2D. originally meant used with a flat map based on UTM coordinates
//turned out to be not accurate enough when spanning multipe UTM zones

//only used for setting zoom level in current implementation

package kripos.math.circle;

import java.awt.geom.Point2D;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

/**
 * Representation of a circle, with methods to calculate numbers relative to
 * another circle.
 * 
 * @author oysteine
 *
 */
public class Circle {

  /**
   * Center of circle;
   */
  public final Point2D origo;
  /**
   * Radius of circle
   */
  public double rad;

  protected int intersectionCount = 0;


  /**
   * 
   * @param x    cartesian x coord
   * @param y    cartesian y coord
   * @param rad  length of radius
   */
  public Circle(double x, double y, double rad) {
    this.origo = new Point2D.Double(x, y);
    this.rad = rad;
  }


  /**
   * 
   * @param angle radians from x axis
   * @return point on circle circumference
   */
  public Point2D getPoint(double angle) {
    double x = origo.getX() + rad * Math.cos(angle);
    double y = origo.getY() + rad * Math.sin(angle);
    return new Point2D.Double(x, y);
  }


  /**
   * @param inter target: a intersection on this circle
   * @return the angle between the x axis and the line from
   *          this circle's origo point to <code>inter</code>'s point 
   */
  double getAngle(Intersection inter) {
    assert this == inter.c1 || this == inter.c2;
    double opposite = inter.point.getY() - origo.getY(); // opposite
    double hyp = rad;
    if (inter.isAbove(origo)) {
      if (inter.isRightOf(origo)) {
        return Math.asin(opposite / hyp);
      } else {
        return Math.PI - Math.asin(opposite / hyp);
      }
    } else {
      if (inter.isRightOf(origo)) {
        return 2 * Math.PI + Math.asin(opposite / hyp);
      } else {
        return Math.PI - Math.asin(opposite / hyp);
      }
    }
  }


  /**
   * 
   * @param inter
   * @return <code>true</code> iff <code>inter</code> is neither inside
   *          this circle's area nor on its circumference
   */
  boolean isOutside(Intersection inter) {
    return !inter.belongsTo(this) &&
    origo.distance(inter.point) > rad;
  }


  /**
   * 
   * @param other
   * @return <code>true</code> iff <code>c</code> is completely within
   *          this circle's area or on its circumference
   */
  boolean isWithin(Circle other) {
    if (other == this) {
      return true;
    }

    double dist = origo.distance(other.origo);
    return dist + other.rad <= rad;
  }


  /**
   * @param other not <code>null</code> 
   * @return intersection points, or <code>null</code> if and only f circles do not intersect
   *  at <i>two</i> points
   */
  List<Intersection> getIntersections(Circle other) {
    assert other != null;

    double maxDist = rad + other.rad;
    double dist = origo.distance(other.origo);
    if (dist >= maxDist) {
      return null;
    }
    if (dist + Math.min(rad,other.rad) < Math.max(rad,other.rad)) {
      return null;
    }

    double x1 = origo.getX();
    double y1 = origo.getY();
    double r1 = rad;
    double x2 = other.origo.getX();
    double y2 = other.origo.getY();
    double r2 = other.rad;

    double d = Math.sqrt(Math.pow(x2-x1,2) + Math.pow(y2-y1,2));

    double ixPart1 = (x2+x1) / 2 + (x2-x1) * (r1*r1-r2*r2) / (2*d*d);
    double ixPart2 = ( (y2-y1) / (2*d*d) ) *
    Math.sqrt((Math.pow(r1+r2,2)-d*d) * (d*d-Math.pow(r2-r1,2))) ;

    double iyPart1 = (y2+y1) / 2 + (y2-y1) * (r1*r1-r2*r2) / (2*d*d);
    double iyPart2 = ( (x2-x1) / (2*d*d) ) *
    Math.sqrt((Math.pow(r1+r2,2)-d*d) * (d*d-Math.pow(r2-r1,2)));

    Point2D first = new Point2D.Double(ixPart1+ixPart2, iyPart1-iyPart2);
    Point2D second = new Point2D.Double(ixPart1-ixPart2, iyPart1+iyPart2);

    List<Intersection> inters = new LinkedList<Intersection>();
    inters.add(new Intersection(first, this, other));
    inters.add(new Intersection(second, this, other));
    return inters;
  }


  /**
   * @param x
   * @param y
   * @return distance from the given point to circumference
   */
  public double getMargin(double x, double y) {
    double margin =
      Math.pow(x - origo.getX(), 2) +
      Math.pow(y - origo.getY(), 2) -
      Math.pow(rad, 2);
    return margin;
  }


  @Override
  public String toString() {
    ByteArrayOutputStream tmp = new ByteArrayOutputStream();
    PrintStream stream = new PrintStream(tmp);
    stream.printf(Locale.US, "Circle(%.2f, %.2f, %.2f)", 
        origo.getX(), origo.getY(), rad);
    return tmp.toString();
  }

}//class