//legacy code. only works in 2D. originally meant used with a flat map based on UTM coordinates
//turned out to be not accurate enough when spanning multipe UTM zones

//only used for setting zoom level in current implementation

package kripos.math.circle;

import java.awt.geom.Point2D;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Calculate intersections from many circles.
 * @author oysteine
 *
 */
public class Intersector {

  /**
   * 
   * @param inters
   * @return length of circumference of convex hull created by <code>inters</code>
   */
  static public double getPolygonLength(List<Intersection> inters) {
    double length = 0.0;

    if (inters.size() == 0) {
    } else if (inters.size() == 1) {
      // circumference of single circle
      Intersection i = inters.get(0);
      assert i.c1 == i.c2;
      return 0.0;
      //return 2.0 * Math.PI * i.c1.rad;
    } else {
      List<Intersection> tmp = new LinkedList<Intersection>(inters);
      Intersection prev = tmp.remove(0);
      tmp.add(prev);
      for (Intersection next : tmp) {
        length += prev.point.distance(next.point);
        prev = next;
      }
    }
    return length;
  }


  /**
   * 
   * @param inters
   * @return area of convex hull created by <code>inters</code>
   */
  static public double getPolygonArea(List<Intersection> inters) {
    if (inters.size() == 0) {
      return 0.0;
    }

    double area = 0.0;

    List<Intersection> tmp = new LinkedList<Intersection>(inters);
    Intersection prev = tmp.remove(0);
    tmp.add(prev);
    for (Intersection next : tmp) {
      double x1 = prev.point.getX();
      double y1 = prev.point.getY();
      double x2 = next.point.getX();
      double y2 = next.point.getY();
      area += x1*y2 - x2*y1;
      prev = next;
    }
    area *= 0.5;
    return area;
  }

  /**
   * @param inters convex hull
   * @return centroid of polygon or <code>null</code> if invalid polygon
   */
  static public Point2D getCentroid(List<Intersection> inters)
  {
    if (inters.size() < 2) {
      return null;
    }

    double area = getPolygonArea(inters);

    double x = 0.0;
    double y = 0.0;


    List<Intersection> tmp = new LinkedList<Intersection>(inters);
    Intersection prev = tmp.remove(0);
    tmp.add(prev);
    for (Intersection next : tmp) {
      double x1 = prev.point.getX();
      double y1 = prev.point.getY();
      double x2 = next.point.getX();
      double y2 = next.point.getY();

      double determinant = x1*y2 - x2*y1;
      x += (x1+x2) * determinant;
      y += (y1+y2) * determinant;

      prev = next;
    }

    x /= (6 * area);
    y /= (6 * area);

    return new Point2D.Double(x,y);
  }


  /**
   * 
   * @param circles
   * @return exact size of area covered by all circles
   */
  static public double getExactArea(List<Circle> circles) {
    if (circles.size() == 1) {
      Circle c = circles.get(0);
      return Math.PI * c.rad * c.rad;
    }

    List<Intersection> inters = getIntersections(circles);
    if (inters.size() == 0) {
      return 0.0;
    }

    double area = getPolygonArea(inters);

    List<Intersection> tmp = new LinkedList<Intersection>(inters);
    Intersection prev = tmp.remove(0);
    tmp.add(prev);
    for (Intersection next : tmp) {
      ArcInfo arc = getArc(prev, next);

      double asize = arc.a2 - arc.a1;
      double sliceArea = asize / 2 * Math.pow(arc.c.rad, 2);
      double triangleArea = Math.pow(arc.c.rad, 2) *
      Math.cos(asize/2) * Math.sin(asize/2);

      area += sliceArea - triangleArea;

      prev = next;
    }

    return area;
  }


  /**
   * 
   * @param inters
   * @param distance  max distance between each point
   * @return all original <code>inters</code> point plus points
   *          located on the circumferences on the circles,
   *          with maximum <code>distance</code> length between
   *          two adjacent points
   */
  public static List<Intersection> getMorePoints(
      List<Intersection> inters, double distance) {
    List<Intersection> populated = new LinkedList<Intersection>();
    if (inters.size() == 0) {
      return populated;
    }

    //System.out.println("GETTING MORE POINTS!");

    List<Intersection> tmp = new LinkedList<Intersection>(inters);
    Intersection prev = tmp.remove(0);
    tmp.add(prev);
    for (Intersection next : tmp) {
      populated.add(prev);

      ArcInfo arc = getArc(prev, next);

      //System.out.printf(arc.c + ": a1=%.2f, a2=%.2f%n", arc.a1, arc.a2);
      double asize = arc.a2 - arc.a1;
      double arclen = arc.c.rad * asize;
      double adelta = asize / (arclen / distance);
      for (double anew = arc.a1 + adelta; anew < arc.a2; anew += adelta) {
        //System.out.printf("  adding %.2f%n", anew);
        Point2D p = arc.c.getPoint(anew);
        populated.add(new Intersection(p, arc.c, arc.c));
      }


      prev = next;
    }

    return populated;
  }


  private static class ArcInfo {
    private final Circle c;
    private final double a1;
    private final double a2;

    private ArcInfo(Circle c, double a1, double a2) {
      this.c = c;
      this.a1 = a1;
      this.a2 = a2;
    }
  }

  private static ArcInfo getArc(Intersection prev, Intersection next) {
    // find potential circles
    Circle c = prev.c1;
    Circle cc = null;
    if (!next.belongsTo(c)) {
      c = prev.c2;
    } else {
      cc = prev.c2;
      if (!next.belongsTo(cc)) {
        cc = null;
      }
    }

    // find degrees and wanted circle

    double a1 = c.getAngle(prev);
    double a2 = c.getAngle(next);
    if (a2 <= a1) { // equals to support a single circle with a single point
      a2 += 2 * Math.PI;
    }

    if (cc != null && cc != c &&
        (a2 - a1 > Math.PI || c.rad < cc.rad)) {
      double b1 = cc.getAngle(prev);
      double b2 = cc.getAngle(next);
      if (b2 < b1) {
        b2 += 2 * Math.PI;
      }

      if (b2 - b1 < Math.PI ||
          cc.rad < c.rad) {
        c = cc;
        a1 = b1;
        a2 = b2;
      }
    }

    return new ArcInfo(c, a1, a2);
  }



  /**
   * 
   * @param circles
   * @return all points where circles cross to make up
   *          the area all circles overlap
   */
  public static List<Intersection> getIntersections(List<Circle> circles) {

    List<Intersection> inters = new LinkedList<Intersection>();

    if (circles.size() == 0) {
      return inters;
    }

    // set intersections at each circle
    List<Circle> sources = new LinkedList<Circle>(circles);
    List<Circle> targets = new LinkedList<Circle>();

    Circle firstSource = sources.remove(0);
    targets.add(firstSource);
    // add dummy intersection
    inters.add(new Intersection(
        firstSource.getPoint(0), firstSource, firstSource));
    firstSource.intersectionCount += 2;

    while (!sources.isEmpty()) {
      Circle source = sources.remove(0);

      // remove previous intersections that fall outside new source
      Iterator<Intersection> iterator = inters.iterator();
      while (iterator.hasNext()) {
        Intersection inter = iterator.next();
        if (source.isOutside(inter)) {
          iterator.remove();
          inter.c1.intersectionCount--;
          inter.c2.intersectionCount--;
        }
      }

      // add new intersections from source
      for (Circle target : targets) {
        List<Intersection> newInters = target.getIntersections(source);

        if (newInters != null) {
          for (Intersection newInter : newInters) {
            boolean valid = true;
            for (Circle target2 : targets) {
              if (target2.isOutside(newInter)) {
                valid = false;
                break;
              }
            }
            if (valid) {
              inters.add(newInter);
              newInter.c1.intersectionCount++;
              newInter.c2.intersectionCount++;
            }
          }
        }
      }

      if (inters.size() == 0) {
        // add dummy intersection if source is within all targets
        boolean withinAll = true;
        for (Circle target : targets) {
          if (!target.isWithin(source)) {
            withinAll = false;
            break;
          }
        }

        // add dummy intersection
        if (withinAll) {
          inters.add(new Intersection(
              source.getPoint(0), source, source));
          source.intersectionCount += 2;
        }
      } else if (inters.size() > 1) {
        // remove dummy intersection
        Intersection test = inters.get(0);
        if (test.c1 == test.c2) {
          inters.remove(0);
          test.c1.intersectionCount -= 2;
        }
      }

      // add source as target
      targets.add(source);

      // remove targets without any intersections
      Iterator<Circle> citer = targets.iterator();
      while (citer.hasNext()) {
        Circle target = citer.next();
        assert target.intersectionCount >= 0 :
          target.toString() + ", intersCount=" + target.intersectionCount;
        if (target.intersectionCount == 0) {
          citer.remove();
        }
      }
    }

    if (inters.size() == 0) {
      return inters;
    }



    // find lowest point to start convex polygon creation
    Intersection bottom = inters.get(0);
    for (Intersection inter : inters) {
      if (inter.isBelow(bottom)) {
        bottom = inter;
      }
    }
    inters.remove(bottom);

    // find angles from bottom to rest of intersections
    AngledIntersection[] angles = new AngledIntersection[inters.size()];
    int i = 0;
    for (Intersection inter : inters) {
      angles[i++] = new AngledIntersection(
          inter,
          bottom.getAngle(inter));
    }

    // sort angles and add
    List<Intersection> sorted = new LinkedList<Intersection>();
    sorted.add(bottom);
    Arrays.sort(angles);
    for (AngledIntersection tmp : angles) {
      sorted.add(tmp.inter);
    }

    return sorted;
  }


  private static class AngledIntersection implements Comparable<AngledIntersection> {
    private final Intersection inter;
    private final double angle;
    private AngledIntersection(Intersection inter, double angle) {
      this.inter = inter;
      this.angle = angle;
    }

    public int compareTo(AngledIntersection other) {
      if (angle < other.angle) {
        return -1;
      } else if (angle > other.angle) {
        return 1;
      } else {
        // TODO: compare distance
        return 0;
      }
    }
  }
}
