package kripos.math.servlet;


import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import kripos.geo.openmap.MapDrawer;

/**
 * Create a map image from the cached circles corresponding to
 * the given ID, using OpenMap.
 * 
 * @author oysteine
 */
public class GetMapImage extends HttpServlet {


  private static final long serialVersionUID = 6720480250134837969L;


  @Override
  public void doGet(HttpServletRequest request, HttpServletResponse response)
  throws IOException, ServletException
  {
    ServletContext sc = getServletContext();

    String ip = request.getParameter("ip");
    String cacheId = request.getParameter("cacheId");
    TraceCacheEntry ce = TraceCache.getCachedTrace(cacheId, ip);
    if (ce == null) {
      sc.log("Could not find cache id '" + cacheId + "' for address '" + ip + "'.");
      response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
      return;
    }


    String strZoom = request.getParameter("zoom");
    String strLongOffset = request.getParameter("longOffset");
    String strLatOffset = request.getParameter("latOffset");
    float zoom = 1.0f;
    if (strZoom != null) {
      zoom = Float.parseFloat(strZoom);
    }
    double longOffset = 0.0;
    if (strLongOffset != null) {
      longOffset = Double.parseDouble(strLongOffset);
    }
    double latOffset = 0.0;
    if (strLatOffset != null) {
      latOffset = Double.parseDouble(strLatOffset);
    }


    byte[] image = getImage(ce, zoom, longOffset, latOffset);
    String filename = "mapView.gif";

    String mimeType = sc.getMimeType(filename);
    if (mimeType == null) {
      sc.log("Could not get MIME type of " + filename);
      response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
      return;
    }

    response.setContentType(mimeType);
    response.setContentLength(image.length);

    OutputStream out = response.getOutputStream();
    out.write(image);
    out.close();
  }

  @Override
  public void doPost(HttpServletRequest request, HttpServletResponse response)
  throws IOException, ServletException
  {
    doGet(request, response);
  }


  private byte[] getImage(TraceCacheEntry ce,
      float zoom, double longOffset, double latOffset) {
    return MapDrawer.getMap(ce.cacheId, ce.circles,
        zoom, latOffset, longOffset);
  }

}