package kripos.tools;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import kripos.geo.Landmark;

/**
 * Creates and fills the database used by agents for 
 * storing geolocation information
 * Any existing database and content will be dropped!
 * 
 * @author oysteine
 * @version
 *
 */
public class DBCreator {
  private String dbmsPath;//TODO not used yet
  private String user;//TODO not used yet
  private String pw;//TODO not used yet
  private String myName;

  /**
   * Creates the database tables relative to the landmark host
   */
  public DBCreator() {
    InetAddress iadr;
    try {
      iadr = InetAddress.getLocalHost();
      myName = iadr.getCanonicalHostName();
    } catch (UnknownHostException e) {
      System.out.println("Unable to get local fqdn hostname");
      e.printStackTrace();
    }
  }

  /**
   * Establishes a connection to the database
   * 
   * @return con connection to the database
   */
  private Connection connect() throws SQLException, ClassNotFoundException{
    Connection c = null;
    Class.forName("org.hsqldb.jdbcDriver");
    c = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "sa", "");
    return c;
  }

  /**
   * Creates and fills the table containing information about landmarks
   * 
   * @param Database connection con
   */
  private void createLandmarkTable(Connection con) throws SQLException{
    String landmarkTable = "CREATE TABLE LANDMARKS " +
    "(NAME VARCHAR(32) NOT NULL, IPADR VARCHAR(39) NOT NULL, CHECKED TIMESTAMP, " +
    "DISTANCE_KM DOUBLE NOT NULL, LATITUDE DOUBLE NOT NULL, LONGITUDE DOUBLE NOT NULL, MIN_RTT DOUBLE, " +
    "AVG_RTT DOUBLE, C1 DOUBLE, EPSILON DOUBLE, HASH VARCHAR(64), PRIMARY KEY(NAME,IPADR))";

    Statement stmt;
    stmt = con.createStatement();
    //create table
    stmt.executeUpdate(landmarkTable);
    //fill table
    LandmarkReader lr = new LandmarkReader();
    ArrayList<Landmark> landmarks = lr.distance(myName);

    for(int i=0;i<landmarks.size();i++){
      Landmark l = landmarks.get(i); 
      double distance = l.getDistance();
      double latitude = l.getGeoPosition().getLatitude();
      double longitude = l.getGeoPosition().getLongitude();
      String IP = l.getIP();
      String name = l.getName();
      String landmarkAdd = "INSERT INTO LANDMARKS " +
      "VALUES ('"+name+"','"+IP+"',null,"+distance+","+latitude+","+longitude+"," +
      9999999+"," + -1+"," + -1+"," + -1+",'A')";

      stmt.executeUpdate(landmarkAdd);
    }
  }

  /**
   * Creates the table containing information about traced hosts
   * 
   * @param Database connection con
   */
  private void createTraceTable(Connection con) throws SQLException{
    //contains traced hosts and info
    String traceTable = "CREATE TABLE TRACED " +
    "(NAME VARCHAR(32), IPADR VARCHAR(39), CHECKED TIMESTAMP NOT NULL, " +
    "MIN_RTT DOUBLE NOT NULL, AVG_RTT DOUBLE, C1 DOUBLE, EPSILON DOUBLE, HASH VARCHAR(64), PRIMARY KEY(IPADR))";

    Statement stmt;
    stmt = con.createStatement();
    stmt.executeUpdate(traceTable);
  }

  /**
   * Creates the table containing misc information, 
   * 
   * @param Database connection con
   */
  private void createMiscTable(Connection con)throws SQLException{
    //contains misc data. when last bestline etc.
    String miscTable = "CREATE TABLE MISC " +
    "(NAME VARCHAR(32), IPADR VARCHAR(39), LAST_BESTLINE TIMESTAMP, BESTLINE_M DOUBLE, BESTLINE_B DOUBLE, LATITUDE DOUBLE, LONGITUDE DOUBLE)";

    Statement stmt = con.createStatement();
    //create table
    stmt.executeUpdate(miscTable);

    //fill table	
    LandmarkReader lr = new LandmarkReader();
    Landmark l = lr.getSingleLandmark(myName);

    double latitude = l.getGeoPosition().getLatitude();
    double longitude = l.getGeoPosition().getLongitude();
    String IP = l.getIP();

    String miscAdd = "INSERT INTO MISC VALUES ('"+myName+"','"+IP+"',null,0,0,"+latitude+","+longitude+")";
    stmt.executeUpdate(miscAdd);
  }

  /**
   * @param args
   * @throws SQLException 
   */
  public static void main(String[] args) {
    DBCreator dbCreate = new DBCreator();
    try{
      Connection con = dbCreate.connect();
      Statement stmt;
      stmt = con.createStatement();
      DatabaseMetaData dbmd = con.getMetaData();
      ResultSet rs1 = dbmd.getTables(null,null,null,null);
      //drop all existing normal tables
      while(rs1.next()){
        String tableName = rs1.getString("TABLE_NAME");
        String tableType = rs1.getString("TABLE_TYPE");
        if(tableType.equalsIgnoreCase("TABLE")){
          stmt.execute("DROP TABLE " +tableName);
        }
      }
      //create and fill tables
      dbCreate.createLandmarkTable(con);
      dbCreate.createTraceTable(con);
      dbCreate.createMiscTable(con);
      con.close();
      System.out.println("Database created successfully!");
      System.exit(0);

    } catch (SQLException se) {
      System.out.println("Database creation FAILED!");
      se.printStackTrace();
      System.exit(1);
    }catch (ClassNotFoundException ce) {
      System.out.println("Database creation FAILED!");
      ce.printStackTrace();
      System.exit(1);
    }
  }

}//class
