package kripos.tools;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.net.InetAddress;
import java.util.ArrayList;
import kripos.geo.Landmark;
import com.bbn.openmap.LatLonPoint;
import com.bbn.openmap.proj.Length;
import com.bbn.openmap.proj.coords.UTMPoint;

/**
 * Tool to read landmarkinformation from file.
 * Includes method to calculate distance between landmarks 
 * 
 * @author oysteine
 * @version 1.1
 *
 */
public class LandmarkReader {
  private ArrayList<Landmark> landmarks = new ArrayList<Landmark>();

  /**
   * Creates a LandmarkReader and reads in landmarkinformation from file  
   */
  public LandmarkReader() {
    convert();
  }

  /**
   * Calculates and returns the distances to all landmarks 
   * from the landmark with the name provided.
   * 
   * @param name of landmark to compute distances from
   * @return list of landmarks with distances.
   */
  public ArrayList<Landmark> distance(String landmarkName){
    Landmark from = null;
    for(int i=0;i<landmarks.size();i++){
      if(landmarkName.equalsIgnoreCase(landmarks.get(i).getName())){
        from = landmarks.remove(i);
        break;
      }
    }

    for(int i=0; i<landmarks.size();i++){
      double radDistance = from.getGeoPosition().distance(landmarks.get(i).getGeoPosition());
      Length converter = Length.KM;
      landmarks.get(i).setDistance(converter.fromRadians(radDistance));
    }
    return landmarks;
  }

  /**
   * Get information about a single landmark in the form of Landmark object
   * 
   * @param landmarkName the name of the
   * @return the Landmark object for the landmark queried for
   */
  public Landmark getSingleLandmark(String landmarkName){
    Landmark result = null;
    for(int i=0;i<landmarks.size();i++){
      if(landmarkName.equalsIgnoreCase(landmarks.get(i).getName())){
        result = landmarks.remove(i);
        break;
      }
    }
    return result;
  }

  /**
   * Reads landmarkinformation from file
   * | is used as field separator
   * lines starting with # are ignored
   */
  private void convert(){		
    try {
      FileReader fr = new FileReader("maalepaaler.txt");
      BufferedReader br = new BufferedReader(fr);
      String temp = null;

      while((temp = br.readLine()) !=null){
        if('#' == temp.charAt(0)){ //ignore lines starting with #
        }
        else{
          int firstCutPoint = temp.indexOf('|');
          String name = temp.substring(0, firstCutPoint);
          int secondCutPoint = temp.indexOf('|', firstCutPoint+1);
          int UTMZone = Integer.parseInt(temp.substring(firstCutPoint+1, secondCutPoint));
          int thirdCutPoint = temp.indexOf('|', secondCutPoint+1);
          int UTMNorthing = Integer.parseInt(temp.substring(secondCutPoint+1, thirdCutPoint));
          int UTMEasting = Integer.parseInt(temp.substring(thirdCutPoint+1));
          UTMPoint utmp = new UTMPoint(UTMNorthing, UTMEasting, UTMZone,'N');
          LatLonPoint llp = utmp.toLatLonPoint();

          //get the current IP-address of the hostname
          InetAddress iadr = InetAddress.getByName(name);
          String IP = iadr.getHostAddress();

          Landmark l = new Landmark(name, llp, IP);
          landmarks.add(l);
        }
      }
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

}//class
